import { CommonError, ICancellationToken, Result } from "@microsoft/powerquery-parser";
import type { DocumentSymbol, FoldingRange, Hover, Location, Position, SignatureHelp } from "vscode-languageserver-types";
import { ParseError, ParseState } from "@microsoft/powerquery-parser/lib/powerquery-parser/parser";
import { AutocompleteItem, TActiveNode, TypeCache } from "../inspection";
import { IDisposable } from "../commonTypes";
import { PartialSemanticToken } from "../providers";
import { TextEdit } from "vscode-languageserver-textdocument";
export interface Analysis extends IDisposable {
    getAutocompleteItems(position: Position, cancellationToken?: ICancellationToken): Promise<Result<AutocompleteItem[] | undefined, CommonError.CommonError>>;
    getDefinition(position: Position, cancellationToken?: ICancellationToken): Promise<Result<Location[] | undefined, CommonError.CommonError>>;
    getDocumentSymbols(cancellationToken?: ICancellationToken): Promise<Result<DocumentSymbol[] | undefined, CommonError.CommonError>>;
    getFoldingRanges(cancellationToken?: ICancellationToken): Promise<Result<FoldingRange[] | undefined, CommonError.CommonError>>;
    getHover(position: Position, cancellationToken?: ICancellationToken): Promise<Result<Hover | undefined, CommonError.CommonError>>;
    getPartialSemanticTokens(cancellationToken?: ICancellationToken): Promise<Result<PartialSemanticToken[] | undefined, CommonError.CommonError>>;
    getSignatureHelp(position: Position, cancellationToken?: ICancellationToken): Promise<Result<SignatureHelp | undefined, CommonError.CommonError>>;
    getRenameEdits(position: Position, newName: string, cancellationToken?: ICancellationToken): Promise<Result<TextEdit[] | undefined, CommonError.CommonError>>;
    getActiveNode(position: Position): Promise<Result<TActiveNode | undefined, CommonError.CommonError>>;
    getParseError(): Promise<Result<ParseError.ParseError | undefined, CommonError.CommonError>>;
    getParseState(): Promise<Result<ParseState | undefined, CommonError.CommonError>>;
    getTypeCache(): TypeCache;
}
