import * as PQP from "@microsoft/powerquery-parser/lib/powerquery-parser";
import { CommonError, ICancellationToken, Result } from "@microsoft/powerquery-parser";
import { DocumentSymbol, FoldingRange, Hover, Location, Position, SignatureHelp } from "vscode-languageserver-types";
import { NodeIdMap, ParseError, ParseState } from "@microsoft/powerquery-parser/lib/powerquery-parser/parser";
import type { TextDocument, TextEdit } from "vscode-languageserver-textdocument";
import { Trace, TraceManager } from "@microsoft/powerquery-parser/lib/powerquery-parser/common/trace";
import { Ast } from "@microsoft/powerquery-parser/lib/powerquery-parser/language";
import { TActiveNode, TypeCache } from "../inspection";
import type { AutocompleteItemProviderContext, DefinitionProviderContext, HoverProviderContext, IAutocompleteItemProvider, ILibraryProvider, ILocalDocumentProvider, PartialSemanticToken, SignatureProviderContext } from "../providers/commonTypes";
import { Inspection, InspectionSettings } from "..";
import type { Analysis } from "./analysis";
import type { AnalysisSettings } from "./analysisSettings";
export declare class AnalysisBase implements Analysis {
    protected textDocument: TextDocument;
    protected initialCorrelationId: number | undefined;
    protected inspectionSettings: InspectionSettings;
    protected locale: string;
    protected traceManager: TraceManager;
    protected languageAutocompleteItemProvider: IAutocompleteItemProvider;
    protected libraryProvider: ILibraryProvider;
    protected localDocumentProvider: ILocalDocumentProvider;
    protected triedLexParse: Promise<PQP.Task.TriedLexParseTask>;
    protected typeCache: TypeCache;
    constructor(textDocument: TextDocument, analysisSettings: AnalysisSettings);
    getAutocompleteItems(position: Position, cancellationTokenOverride?: ICancellationToken | undefined): Promise<Result<Inspection.AutocompleteItem[] | undefined, CommonError.CommonError>>;
    getDefinition(position: Position, cancellationTokenOverride?: ICancellationToken | undefined): Promise<Result<Location[] | undefined, CommonError.CommonError>>;
    getDocumentSymbols(cancellationTokenOverride?: ICancellationToken | undefined): Promise<PQP.Result<DocumentSymbol[] | undefined, CommonError.CommonError>>;
    getFoldingRanges(cancellationTokenOverride?: ICancellationToken | undefined): Promise<Result<FoldingRange[] | undefined, CommonError.CommonError>>;
    getHover(position: Position, cancellationTokenOverride?: ICancellationToken | undefined): Promise<Result<Hover | undefined, CommonError.CommonError>>;
    getActiveNode(position: Position): Promise<Result<TActiveNode | undefined, CommonError.CommonError>>;
    getParseError(): Promise<Result<ParseError.ParseError | undefined, CommonError.CommonError>>;
    getParseState(): Promise<Result<ParseState | undefined, CommonError.CommonError>>;
    getPartialSemanticTokens(cancellationTokenOverride?: ICancellationToken | undefined): Promise<Result<PartialSemanticToken[] | undefined, CommonError.CommonError>>;
    getSignatureHelp(position: Position, cancellationTokenOverride?: ICancellationToken | undefined): Promise<Result<SignatureHelp | undefined, CommonError.CommonError>>;
    getRenameEdits(position: Position, newName: string, cancellationTokenOverride?: ICancellationToken | undefined): Promise<Result<TextEdit[] | undefined, CommonError.CommonError>>;
    dispose(): void;
    protected static isValidHoverIdentifier(activeNode: Inspection.ActiveNode): boolean;
    protected getAutocompleteItemProviderContext(position: Position, correlationId: number, cancellationTokenOverride: ICancellationToken | undefined): Promise<AutocompleteItemProviderContext | undefined>;
    protected getDefinitionProviderContext(position: Position, correlationId: number, cancellationTokenOverride: ICancellationToken | undefined): Promise<DefinitionProviderContext | undefined>;
    protected getHoverProviderContext(position: Position, correlationId: number, cancellationTokenOverride: ICancellationToken | undefined): Promise<HoverProviderContext | undefined>;
    protected getSignatureProviderContext(position: Position, correlationId: number, cancellationTokenOverride: ICancellationToken | undefined): Promise<SignatureProviderContext | undefined>;
    getTypeCache(): TypeCache;
    protected collectAllIdentifiersBeneath(nodeIdMapCollection: NodeIdMap.Collection, valueCreator: Ast.Identifier | Ast.GeneralizedIdentifier): Promise<Array<Ast.Identifier | Ast.GeneralizedIdentifier>>;
    protected tryLexParse(): Promise<PQP.Task.TriedLexParseTask>;
    protected inspectAutocomplete(activeNode: TActiveNode, correlationId: number, cancellationTokenOverride: ICancellationToken | undefined): Promise<Inspection.Autocomplete | undefined>;
    protected inspectCurrentInvokeExpression(position: Position, correlationId: number, cancellationTokenOverride: ICancellationToken | undefined): Promise<Inspection.TriedCurrentInvokeExpression | undefined>;
    protected inspectNodeScope(activeNode: TActiveNode, correlationId: number, cancellationTokenOverride: ICancellationToken | undefined): Promise<Inspection.TriedNodeScope | undefined>;
    protected inspectScopeType(activeNode: TActiveNode, correlationId: number, cancellationTokenOverride: ICancellationToken | undefined): Promise<Inspection.TriedScopeType | undefined>;
    protected getActiveNodeOkOrThrow(position: Position, trace: Trace | undefined): Promise<TActiveNode | undefined>;
    protected getParseErrorOkOrThrow(trace: Trace | undefined): Promise<ParseError.ParseError | undefined>;
    protected getParseStateOkOrThrow(trace: Trace | undefined): Promise<ParseState | undefined>;
    protected applyAnalysisSettings(analysisSettings: AnalysisSettings): void;
}
