"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.analysis = void 0;
const documentAnalysis_1 = require("./documentAnalysis");
const analysisByUri = new Map();
// If AnalysisSettings.isWorkspaceCacheAllowed is true then the analysis will be cached by its URI.
// A URI with an updated version will cause a new Analysis to be generated and cached.
function analysis(document, analysisSettings) {
    const cacheKey = document.uri.toString();
    if (analysisSettings.isWorkspaceCacheAllowed && analysisByUri.has(cacheKey)) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const [analysis, version] = analysisByUri.get(cacheKey);
        if (version === document.version) {
            return analysis;
        }
        else {
            analysisByUri.delete(cacheKey);
        }
    }
    const analysis = new documentAnalysis_1.DocumentAnalysis(document, analysisSettings);
    if (analysisSettings.isWorkspaceCacheAllowed) {
        analysisByUri.set(cacheKey, [analysis, document.version]);
    }
    return analysis;
}
exports.analysis = analysis;
//# sourceMappingURL=analysisUtils.js.map