"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MemoizedAnalysis = void 0;
const inspection_1 = require("../inspection");
const analysisBase_1 = require("./analysisBase");
// Adds caching to AnalysisBase.
class MemoizedAnalysis extends analysisBase_1.AnalysisBase {
    constructor() {
        super(...arguments);
        this.activeNodeCache = new Map();
        this.autocompleteCache = new Map();
        this.currentInvokeExpressionCache = new Map();
        this.nodeScopeCache = new Map();
        this.scopeTypeCache = new Map();
    }
    dispose() {
        this.activeNodeCache.clear();
        this.autocompleteCache.clear();
        this.currentInvokeExpressionCache.clear();
        this.nodeScopeCache.clear();
        this.scopeTypeCache.clear();
        super.dispose();
    }
    getActiveNode(position) {
        return this.getOrCreate(this.activeNodeCache, () => `${position.line}:${position.character}`, () => super.getActiveNode(position));
    }
    inspectAutocomplete(activeNode, correlationId, cancellationToken) {
        return this.getOrCreate(this.autocompleteCache, () => { var _a, _b; return (inspection_1.ActiveNodeUtils.isPositionInBounds(activeNode) ? (_b = (_a = activeNode.ancestry[0]) === null || _a === void 0 ? void 0 : _a.node) === null || _b === void 0 ? void 0 : _b.id : undefined); }, () => super.inspectAutocomplete(activeNode, correlationId, cancellationToken));
    }
    inspectCurrentInvokeExpression(position, correlationId, cancellationToken) {
        return this.getOrCreate(this.currentInvokeExpressionCache, () => `${position.line}:${position.character}`, () => super.inspectCurrentInvokeExpression(position, correlationId, cancellationToken));
    }
    inspectNodeScope(activeNode, correlationId, cancellationToken) {
        return this.getOrCreate(this.nodeScopeCache, () => { var _a, _b; return (inspection_1.ActiveNodeUtils.isPositionInBounds(activeNode) ? (_b = (_a = activeNode.ancestry[0]) === null || _a === void 0 ? void 0 : _a.node) === null || _b === void 0 ? void 0 : _b.id : undefined); }, () => super.inspectNodeScope(activeNode, correlationId, cancellationToken));
    }
    inspectScopeType(activeNode, correlationId, cancellationToken) {
        return this.getOrCreate(this.scopeTypeCache, () => { var _a, _b; return (inspection_1.ActiveNodeUtils.isPositionInBounds(activeNode) ? (_b = (_a = activeNode.ancestry[0]) === null || _a === void 0 ? void 0 : _a.node) === null || _b === void 0 ? void 0 : _b.id : undefined); }, () => super.inspectScopeType(activeNode, correlationId, cancellationToken));
    }
    // Assumes the first call to getOrCreate will create the cache entry,
    // and any subsequent calls will return the cached value.
    getOrCreate(cache, cacheKeyFactory, valueFactory) {
        return __awaiter(this, void 0, void 0, function* () {
            const cacheKey = cacheKeyFactory();
            let result = cache.get(cacheKey);
            if (result !== undefined) {
                return yield result;
            }
            result = valueFactory();
            cache.set(cacheKey, result);
            return result;
        });
    }
}
exports.MemoizedAnalysis = MemoizedAnalysis;
//# sourceMappingURL=memoizedAnalysis.js.map