"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryFormat = void 0;
const PQF = require("@microsoft/powerquery-formatter");
const PQP = require("@microsoft/powerquery-parser");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
function tryFormat(document, settings) {
    return powerquery_parser_1.ResultUtils.ensureResultAsync(() => __awaiter(this, void 0, void 0, function* () {
        const formatSettings = Object.assign(Object.assign({}, PQF.DefaultSettings), settings);
        const triedFormat = yield PQF.tryFormat(formatSettings, document.getText());
        if (powerquery_parser_1.ResultUtils.isOk(triedFormat)) {
            return [vscode_languageserver_types_1.TextEdit.replace(fullDocumentRange(document), triedFormat.value)];
        }
        // If an unhandled exception was returned.
        else if (PQP.CommonError.isCommonError(triedFormat.error)) {
            throw triedFormat;
        }
        // Else a lexer or parser error was returned and should be ignored.
        else {
            return undefined;
        }
    }), settings.locale);
}
exports.tryFormat = tryFormat;
// TODO: is there a better way to do this?
function fullDocumentRange(document) {
    return {
        start: document.positionAt(0),
        end: {
            line: document.lineCount - 1,
            character: Number.MAX_VALUE,
        },
    };
}
//# sourceMappingURL=formatter.js.map