"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.comparer = exports.fromScopeItem = exports.fromLibraryDefinition = exports.fromKeywordKind = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const jaroWinkler_1 = require("../../../jaroWinkler");
const __1 = require("../../..");
function fromKeywordKind(label, other) {
    const jaroWinklerScore = other !== undefined ? (0, jaroWinkler_1.calculateJaroWinkler)(label, other) : 1;
    return {
        jaroWinklerScore,
        kind: vscode_languageserver_types_1.CompletionItemKind.Keyword,
        label,
        powerQueryType: language_1.Type.NotApplicableInstance,
    };
}
exports.fromKeywordKind = fromKeywordKind;
function fromLibraryDefinition(label, libraryDefinition, other) {
    const jaroWinklerScore = other !== undefined ? (0, jaroWinkler_1.calculateJaroWinkler)(label, other) : 1;
    return {
        jaroWinklerScore,
        kind: libraryDefinition.completionItemKind,
        label,
        powerQueryType: libraryDefinition.asPowerQueryType,
    };
}
exports.fromLibraryDefinition = fromLibraryDefinition;
function fromScopeItem(label, scopeItem, powerQueryType, other) {
    switch (scopeItem.kind) {
        case __1.Inspection.ScopeItemKind.LetVariable:
        case __1.Inspection.ScopeItemKind.RecordField:
        case __1.Inspection.ScopeItemKind.SectionMember:
            if (scopeItem.value === undefined) {
                return undefined;
            }
            break;
        case __1.Inspection.ScopeItemKind.Each:
            return undefined;
        case __1.Inspection.ScopeItemKind.Parameter: {
            break;
        }
        case __1.Inspection.ScopeItemKind.Undefined: {
            if (parser_1.XorNodeUtils.isContext(scopeItem.xorNode)) {
                return undefined;
            }
            break;
        }
        default:
            throw powerquery_parser_1.Assert.isNever(scopeItem);
    }
    const jaroWinklerScore = other !== undefined ? (0, jaroWinkler_1.calculateJaroWinkler)(label, other) : 1;
    return {
        jaroWinklerScore,
        kind: vscode_languageserver_types_1.CompletionItemKind.Variable,
        label,
        powerQueryType,
    };
}
exports.fromScopeItem = fromScopeItem;
function comparer(left, right) {
    const jaroWinklerScoreDiff = right.jaroWinklerScore - left.jaroWinklerScore;
    if (jaroWinklerScoreDiff !== 0) {
        return jaroWinklerScoreDiff;
    }
    else if (left.label < right.label) {
        return -1;
    }
    else if (left.label > right.label) {
        return 1;
    }
    else {
        return 0;
    }
}
exports.comparer = comparer;
//# sourceMappingURL=autocompleteItemUtils.js.map