"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocompleteKeyword = exports.tryAutocompleteKeyword = void 0;
const PQP = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const activeNode_1 = require("../../activeNode");
const autocompleteItem_1 = require("../autocompleteItem");
const autocompleteKeywordDefault_1 = require("./autocompleteKeywordDefault");
const autocompleteKeywordErrorHandlingExpression_1 = require("./autocompleteKeywordErrorHandlingExpression");
const autocompleteKeywordIdentifierPairedExpression_1 = require("./autocompleteKeywordIdentifierPairedExpression");
const autocompleteKeywordLetExpression_1 = require("./autocompleteKeywordLetExpression");
const autocompleteKeywordListExpression_1 = require("./autocompleteKeywordListExpression");
const autocompleteKeywordSectionMember_1 = require("./autocompleteKeywordSectionMember");
const autocompleteKeywordTrailingText_1 = require("./autocompleteKeywordTrailingText");
const __1 = require("../../..");
function tryAutocompleteKeyword(settings, nodeIdMapCollection, activeNode, trailingToken) {
    if (!activeNode_1.ActiveNodeUtils.isPositionInBounds(activeNode)) {
        return Promise.resolve(powerquery_parser_1.ResultUtils.ok([
            ...language_1.Keyword.ExpressionKeywordKinds.map((keywordKind) => autocompleteItem_1.AutocompleteItemUtils.fromKeywordKind(keywordKind)),
            autocompleteItem_1.AutocompleteItemUtils.fromKeywordKind(language_1.Keyword.KeywordKind.Section),
        ]));
    }
    return powerquery_parser_1.ResultUtils.ensureResultAsync(() => autocompleteKeyword(nodeIdMapCollection, activeNode, trailingToken), settings.locale);
}
exports.tryAutocompleteKeyword = tryAutocompleteKeyword;
function autocompleteKeyword(nodeIdMapCollection, activeNode, trailingToken) {
    return __awaiter(this, void 0, void 0, function* () {
        const ancestryLeaf = activeNode_1.ActiveNodeUtils.assertGetLeaf(activeNode);
        let positionName;
        if (__1.PositionUtils.isInXor(nodeIdMapCollection, activeNode.position, ancestryLeaf, false, true)) {
            if (activeNode.exclusiveIdentifierUnderPosition !== undefined) {
                positionName = activeNode.exclusiveIdentifierUnderPosition.normalizedLiteral;
            }
            // Matches 'null', 'true', and 'false'.
            else if (parser_1.XorNodeUtils.isAstChecked(ancestryLeaf, language_1.Ast.NodeKind.LiteralExpression) &&
                (ancestryLeaf.node.literalKind === language_1.Ast.LiteralKind.Logical ||
                    ancestryLeaf.node.literalKind === language_1.Ast.LiteralKind.Null)) {
                positionName = ancestryLeaf.node.literal;
            }
        }
        if (activeNode.ancestry.length < 2) {
            return Promise.resolve(createAutocompleteItems(handleConjunctions(activeNode, [], trailingToken), positionName));
        }
        const edgeCase = findEdgeCase(activeNode, trailingToken);
        if (edgeCase !== undefined) {
            return Promise.resolve(edgeCase);
        }
        const state = {
            nodeIdMapCollection,
            activeNode,
            trailingToken,
            parent: activeNode.ancestry[1],
            child: activeNode_1.ActiveNodeUtils.assertGetLeaf(activeNode),
            ancestryIndex: 0,
        };
        const keywordKinds = yield traverseAncestors(state);
        const keywordsSubset = handleConjunctions(state.activeNode, keywordKinds, trailingToken);
        return Promise.resolve(createAutocompleteItems(keywordsSubset, positionName));
    });
}
exports.autocompleteKeyword = autocompleteKeyword;
const ConjunctionKeywords = [
    language_1.Keyword.KeywordKind.And,
    language_1.Keyword.KeywordKind.As,
    language_1.Keyword.KeywordKind.Is,
    language_1.Keyword.KeywordKind.Meta,
    language_1.Keyword.KeywordKind.Or,
];
// Travel the ancestry path in Active node in [parent, child] pairs.
// Without zipping the values we wouldn't know what we're completing for.
// For example 'if true |' gives us a pair something like [IfExpression, Constant].
// We can now know we failed to parse a 'then' constant.
function traverseAncestors(state) {
    return __awaiter(this, void 0, void 0, function* () {
        const ancestry = state.activeNode.ancestry;
        const numNodes = ancestry.length;
        let inspected;
        for (let ancestryIndex = 1; ancestryIndex < numNodes; ancestryIndex += 1) {
            state.ancestryIndex = ancestryIndex;
            state.parent = ancestry[ancestryIndex];
            state.child = ancestry[ancestryIndex - 1];
            switch (state.parent.node.kind) {
                case language_1.Ast.NodeKind.ErrorHandlingExpression:
                    inspected = (0, autocompleteKeywordErrorHandlingExpression_1.autocompleteKeywordErrorHandlingExpression)(state);
                    break;
                case language_1.Ast.NodeKind.IdentifierPairedExpression:
                    inspected = (0, autocompleteKeywordIdentifierPairedExpression_1.autocompleteKeywordIdentifierPairedExpression)(state);
                    break;
                case language_1.Ast.NodeKind.LetExpression:
                    // eslint-disable-next-line no-await-in-loop
                    inspected = yield (0, autocompleteKeywordLetExpression_1.autocompleteKeywordLetExpression)(state);
                    break;
                case language_1.Ast.NodeKind.ListExpression:
                    inspected = (0, autocompleteKeywordListExpression_1.autocompleteKeywordListExpression)(state);
                    break;
                case language_1.Ast.NodeKind.SectionMember:
                    // eslint-disable-next-line no-await-in-loop
                    inspected = yield (0, autocompleteKeywordSectionMember_1.autocompleteKeywordSectionMember)(state);
                    break;
                default:
                    inspected = (0, autocompleteKeywordDefault_1.autocompleteKeywordDefault)(state);
            }
            if (inspected !== undefined) {
                return inspected;
            }
        }
        return [];
    });
}
function findEdgeCase(activeNode, trailingToken) {
    const ancestry = activeNode.ancestry;
    let inspected;
    // The user is typing in a new file, which the parser defaults to searching for an identifier.
    // `l|` -> `let`
    if (trailingToken === undefined &&
        ancestry.length === 2 &&
        parser_1.XorNodeUtils.isAstChecked(ancestry[0], language_1.Ast.NodeKind.Identifier) &&
        parser_1.XorNodeUtils.isNodeKind(ancestry[1], language_1.Ast.NodeKind.IdentifierExpression)) {
        const identifier = ancestry[0].node.literal;
        inspected = language_1.Keyword.StartOfDocumentKeywords.map((keywordKind) => autocompleteItem_1.AutocompleteItemUtils.fromKeywordKind(keywordKind, identifier));
    }
    // `(x |) => x+1` -> `(x as|) => x+1`
    else if (parser_1.XorNodeUtils.isAstChecked(ancestry[0], language_1.Ast.NodeKind.Identifier) &&
        parser_1.XorNodeUtils.isNodeKind(ancestry[1], language_1.Ast.NodeKind.Parameter) &&
        activeNode.leafKind === activeNode_1.ActiveNodeLeafKind.IsBeforePosition) {
        inspected = [autocompleteItem_1.AutocompleteItemUtils.fromKeywordKind(language_1.Keyword.KeywordKind.As)];
    }
    // `(foo a|) => foo` -> `(foo as) => foo
    else if ((trailingToken === null || trailingToken === void 0 ? void 0 : trailingToken.data) === "a" &&
        parser_1.XorNodeUtils.isContextChecked(ancestry[0], language_1.Ast.NodeKind.Constant) &&
        parser_1.XorNodeUtils.isNodeKind(ancestry[1], language_1.Ast.NodeKind.ParameterList) &&
        parser_1.XorNodeUtils.isNodeKind(ancestry[2], language_1.Ast.NodeKind.FunctionExpression)) {
        inspected = [autocompleteItem_1.AutocompleteItemUtils.fromKeywordKind(language_1.Keyword.KeywordKind.As)];
    }
    return inspected;
}
function createAutocompleteItems(keywordKinds, positionName) {
    return keywordKinds.map((kind) => autocompleteItem_1.AutocompleteItemUtils.fromKeywordKind(kind, positionName));
}
function handleConjunctions(activeNode, inspected, trailingToken) {
    if (activeNode.leafKind !== activeNode_1.ActiveNodeLeafKind.IsBeforePosition &&
        activeNode.leafKind !== activeNode_1.ActiveNodeLeafKind.ContextNode) {
        return inspected;
    }
    // Might be a section document.
    // `[x=1] s`
    // `[x=1] |`
    if (activeNode.ancestry.length === 2 &&
        parser_1.XorNodeUtils.isAstChecked(activeNode.ancestry[1], language_1.Ast.NodeKind.RecordExpression)) {
        if (trailingToken === undefined) {
            return PQP.ArrayUtils.concatUnique(inspected, [language_1.Keyword.KeywordKind.Section]);
        }
        else if (trailingToken.kind === PQP.Language.Token.TokenKind.Identifier &&
            __1.PositionUtils.isInToken(activeNode.position, trailingToken, true, true)) {
            return (0, autocompleteKeywordTrailingText_1.autocompleteKeywordTrailingText)(inspected, trailingToken, [language_1.Keyword.KeywordKind.Section]);
        }
    }
    const activeNodeLeaf = activeNode_1.ActiveNodeUtils.assertGetLeaf(activeNode);
    // `let x = 1 a|`
    if (trailingToken !== undefined && trailingToken.isPositionEitherInOrOnToken) {
        return (0, autocompleteKeywordTrailingText_1.autocompleteKeywordTrailingText)(inspected, trailingToken, undefined);
    }
    // `let x = |`
    // `let x = 1|`
    // `let x = 1 | a`
    else if (parser_1.XorNodeUtils.isTUnaryExpression(activeNodeLeaf)) {
        // `let x = 1 | a`
        if (trailingToken !== undefined &&
            __1.PositionUtils.isAfterTokenPosition(activeNode.position, trailingToken.positionStart, false)) {
            return inspected;
        }
        // `let x = 1|`
        else if (parser_1.XorNodeUtils.isAst(activeNodeLeaf)) {
            return PQP.ArrayUtils.concatUnique(inspected, ConjunctionKeywords);
        }
        // `let x = |`
        else {
            return inspected;
        }
    }
    else {
        return inspected;
    }
}
//# sourceMappingURL=autocompleteKeyword.js.map