"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocompleteKeywordDefault = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const activeNode_1 = require("../../activeNode");
const __1 = require("../../..");
function autocompleteKeywordDefault(state) {
    const activeNode = state.activeNode;
    const child = state.child;
    const key = createMapKey(state.parent.node.kind, child.node.attributeIndex);
    if (AutocompleteExpressionKeys.indexOf(key) !== -1) {
        return autocompleteDefaultExpression(state);
    }
    else {
        const mappedKeywordKind = AutocompleteConstantMap.get(key);
        return mappedKeywordKind !== undefined
            ? autocompleteKeywordConstant(activeNode, child, mappedKeywordKind)
            : undefined;
    }
}
exports.autocompleteKeywordDefault = autocompleteKeywordDefault;
const AutocompleteExpressionKeys = [
    createMapKey(language_1.Ast.NodeKind.ErrorRaisingExpression, 1),
    createMapKey(language_1.Ast.NodeKind.GeneralizedIdentifierPairedExpression, 2),
    createMapKey(language_1.Ast.NodeKind.FunctionExpression, 3),
    createMapKey(language_1.Ast.NodeKind.IdentifierPairedExpression, 2),
    createMapKey(language_1.Ast.NodeKind.IfExpression, 1),
    createMapKey(language_1.Ast.NodeKind.IfExpression, 3),
    createMapKey(language_1.Ast.NodeKind.IfExpression, 5),
    createMapKey(language_1.Ast.NodeKind.InvokeExpression, 1),
    createMapKey(language_1.Ast.NodeKind.LetExpression, 3),
    createMapKey(language_1.Ast.NodeKind.ListExpression, 1),
    createMapKey(language_1.Ast.NodeKind.OtherwiseExpression, 1),
    createMapKey(language_1.Ast.NodeKind.ParenthesizedExpression, 1),
];
// If we're coming from a constant then we can quickly evaluate using a map.
// This is possible because reading a Constant is binary.
// Either the Constant was read and you're in the next context, or you didn't and you're in the constant's context.
const AutocompleteConstantMap = new Map([
    // Ast.NodeKind.ErrorRaisingExpression
    [createMapKey(language_1.Ast.NodeKind.ErrorRaisingExpression, 0), language_1.Keyword.KeywordKind.Error],
    // Ast.NodeKind.IfExpression
    [createMapKey(language_1.Ast.NodeKind.IfExpression, 0), language_1.Keyword.KeywordKind.If],
    [createMapKey(language_1.Ast.NodeKind.IfExpression, 2), language_1.Keyword.KeywordKind.Then],
    [createMapKey(language_1.Ast.NodeKind.IfExpression, 4), language_1.Keyword.KeywordKind.Else],
    // Ast.NodeKind.LetExpression
    [createMapKey(language_1.Ast.NodeKind.LetExpression, 2), language_1.Keyword.KeywordKind.In],
    // Ast.NodeKind.OtherwiseExpression
    [createMapKey(language_1.Ast.NodeKind.OtherwiseExpression, 0), language_1.Keyword.KeywordKind.Otherwise],
    // Ast.NodeKind.Section
    [createMapKey(language_1.Ast.NodeKind.Section, 1), language_1.Keyword.KeywordKind.Section],
]);
function autocompleteDefaultExpression(state) {
    const activeNode = state.activeNode;
    const child = state.child;
    // '[x=|1]
    if (activeNode.leafKind === activeNode_1.ActiveNodeLeafKind.IsInAst) {
        return language_1.Keyword.ExpressionKeywordKinds;
    }
    // `if 1|`
    else if (parser_1.XorNodeUtils.isAstChecked(child, language_1.Ast.NodeKind.LiteralExpression) &&
        child.node.literalKind === language_1.Ast.LiteralKind.Numeric) {
        return [];
    }
    return language_1.Keyword.ExpressionKeywordKinds;
}
function autocompleteKeywordConstant(activeNode, child, keywordKind) {
    if (__1.PositionUtils.isBeforeXor(activeNode.position, child, false)) {
        return undefined;
    }
    else if (parser_1.XorNodeUtils.isAst(child)) {
        // So long as you're inside of an Ast Constant there's nothing that can be recommended other than the constant.
        // Note that we previously checked isBeforeXorNode so we can use the quicker isOnAstNodeEnd to check
        // if we're inside of the Ast node.
        return __1.PositionUtils.isOnAstEnd(activeNode.position, child.node) ? [] : [keywordKind];
    }
    return [keywordKind];
}
// A tuple can't easily be used as a Map key as it does a shallow comparison.
// The work around is to stringify the tuple key, even though we lose typing by doing so.
// [parent XorNode.node.kind, child XorNode.node.attributeIndex].join(",")
function createMapKey(nodeKind, attributeIndex) {
    return [nodeKind, attributeIndex].join(",");
}
//# sourceMappingURL=autocompleteKeywordDefault.js.map