"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocompleteKeywordErrorHandlingExpression = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const __1 = require("../../..");
function autocompleteKeywordErrorHandlingExpression(state) {
    const position = state.activeNode.position;
    const child = state.child;
    const trailingText = state.trailingToken;
    const childAttributeIndex = child.node.attributeIndex;
    if (childAttributeIndex === 0) {
        return [language_1.Keyword.KeywordKind.Try];
    }
    else if (childAttributeIndex === 1) {
        // 'try true o|' creates a ParseError.
        // It's ambiguous if the next token should be either 'otherwise', 'or', or 'catch'.
        if (trailingText !== undefined) {
            const trailingToken = trailingText;
            // First we test if we can autocomplete using the error token.
            if (trailingToken.kind === language_1.Token.TokenKind.Identifier &&
                __1.PositionUtils.isInToken(position, trailingToken, false, true)) {
                const tokenData = trailingText.data;
                // If we can exclude 'or' then the only thing we can autocomplete is 'otherwise'.
                if (tokenData.length > 1 && language_1.Keyword.KeywordKind.Otherwise.startsWith(tokenData)) {
                    return [language_1.Keyword.KeywordKind.Otherwise];
                }
                // In the ambiguous case we don't know what they're typing yet, so we suggest both.
                // In the case of an identifier that doesn't match a 'or' or 'otherwise'
                // we still suggest the only valid keywords allowed.
                // In both cases the return is the same.
                else {
                    return [language_1.Keyword.KeywordKind.Or, language_1.Keyword.KeywordKind.Otherwise];
                }
            }
            // There exists an error token we can't map it to an OtherwiseExpression.
            else {
                return undefined;
            }
        }
        else if (parser_1.XorNodeUtils.isAst(child) && __1.PositionUtils.isAfterAst(position, child.node, true)) {
            return [language_1.Keyword.KeywordKind.Otherwise];
        }
        else {
            return language_1.Keyword.ExpressionKeywordKinds;
        }
    }
    else {
        return undefined;
    }
}
exports.autocompleteKeywordErrorHandlingExpression = autocompleteKeywordErrorHandlingExpression;
//# sourceMappingURL=autocompleteKeywordErrorHandlingExpression.js.map