"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocompleteKeywordLetExpression = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const autocompleteKeywordDefault_1 = require("./autocompleteKeywordDefault");
const common_1 = require("./common");
const autocompleteKeywordTrailingText_1 = require("./autocompleteKeywordTrailingText");
const __1 = require("../../..");
function autocompleteKeywordLetExpression(state) {
    return __awaiter(this, void 0, void 0, function* () {
        // LetExpressions can trigger another inspection which will always hit the same LetExpression.
        // Make sure that it doesn't trigger an infinite recursive call.
        const child = state.child;
        let inspected;
        // Might be either `in` or whatever the autocomplete is for the the last child of the variableList.
        // `let x = 1 |`
        if (child.node.attributeIndex === 2 && parser_1.XorNodeUtils.isContext(child)) {
            inspected = yield autocompleteLastKeyValuePair(state, parser_1.NodeIdMapIterator.iterLetExpression(state.nodeIdMapCollection, parser_1.XorNodeUtils.assertAsNodeKind(state.parent, language_1.Ast.NodeKind.LetExpression)));
            if (state.trailingToken !== undefined) {
                if (state.trailingToken.isPositionEitherInOrOnToken) {
                    // We don't want inspected to be zero legnth.
                    // It's either undefined or non-zero length.
                    inspected = (0, autocompleteKeywordTrailingText_1.autocompleteKeywordTrailingText)(inspected !== null && inspected !== void 0 ? inspected : [], state.trailingToken, [
                        language_1.Keyword.KeywordKind.In,
                    ]);
                    return inspected.length ? inspected : undefined;
                }
                else if (__1.PositionUtils.isBeforeTokenPosition(state.activeNode.position, state.trailingToken.positionStart, true)) {
                    return inspected !== undefined ? [...inspected, language_1.Keyword.KeywordKind.In] : inspected;
                }
            }
            else {
                return inspected !== undefined ? [...inspected, language_1.Keyword.KeywordKind.In] : [language_1.Keyword.KeywordKind.In];
            }
        }
        // `let foo = e|` where we want to treat the identifier as a potential keyword
        else if (child.node.attributeIndex === 1) {
            const identifier = parser_1.AncestryUtils.nth(state.activeNode.ancestry, state.ancestryIndex - 5);
            if (identifier && parser_1.XorNodeUtils.isAstChecked(identifier, language_1.Ast.NodeKind.Identifier)) {
                const identifierAst = identifier.node;
                inspected = language_1.Keyword.ExpressionKeywordKinds.filter((value) => value.startsWith(identifierAst.literal));
            }
        }
        return inspected !== null && inspected !== void 0 ? inspected : (0, autocompleteKeywordDefault_1.autocompleteKeywordDefault)(state);
    });
}
exports.autocompleteKeywordLetExpression = autocompleteKeywordLetExpression;
function autocompleteLastKeyValuePair(state, keyValuePairs) {
    if (keyValuePairs.length === 0) {
        return Promise.resolve(undefined);
    }
    // Grab the last value (if one exists)
    const lastValue = keyValuePairs[keyValuePairs.length - 1].value;
    if (lastValue === undefined) {
        return Promise.resolve(undefined);
    }
    return (0, common_1.autocompleteKeywordRightMostLeaf)(state, lastValue.node.id);
}
//# sourceMappingURL=autocompleteKeywordLetExpression.js.map