"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocompleteKeywordListExpression = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const __1 = require("../../..");
function autocompleteKeywordListExpression(state) {
    const activeNode = state.activeNode;
    const ancestryIndex = state.ancestryIndex;
    const child = state.child;
    // '{' or '}'
    if (child.node.attributeIndex === 0 || child.node.attributeIndex === 2) {
        return undefined;
    }
    powerquery_parser_1.Assert.isTrue(child.node.attributeIndex === 1, `must be in range [0, 2]`, {
        nodeId: child.node.id,
        attributeIndex: child.node.attributeIndex,
    });
    // ListExpression -> ArrayWrapper -> Csv -> X
    const nodeOrComma = parser_1.AncestryUtils.assertNth(activeNode.ancestry, ancestryIndex - 3);
    if (nodeOrComma.node.attributeIndex !== 0) {
        return undefined;
    }
    // We know it's the node component of the Csv,
    // but we have to drill down one more level if it's a RangeExpression.
    const itemNode = parser_1.XorNodeUtils.isNodeKind(nodeOrComma, language_1.Ast.NodeKind.RangeExpression)
        ? parser_1.AncestryUtils.assertNth(activeNode.ancestry, ancestryIndex - 4)
        : nodeOrComma;
    if (parser_1.XorNodeUtils.isContext(itemNode) || __1.PositionUtils.isBeforeXor(activeNode.position, itemNode, false)) {
        return language_1.Keyword.ExpressionKeywordKinds;
    }
    else {
        return undefined;
    }
}
exports.autocompleteKeywordListExpression = autocompleteKeywordListExpression;
//# sourceMappingURL=autocompleteKeywordListExpression.js.map