"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocompleteKeywordSectionMember = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const common_1 = require("./common");
function autocompleteKeywordSectionMember(state) {
    const childAttributeIndex = state.child.node.attributeIndex;
    // SectionMember.namePairedExpression
    if (childAttributeIndex === 2) {
        // A test for 'shared', which as we're on namePairedExpression we either parsed it or skipped it.
        const sharedConstant = parser_1.NodeIdMapUtils.nthChildXorChecked(state.nodeIdMapCollection, state.parent.node.id, 1, language_1.Ast.NodeKind.Constant);
        // 'shared' was parsed so we can exit.
        if (sharedConstant !== undefined) {
            return Promise.resolve(undefined);
        }
        // SectionMember -> IdentifierPairedExpression -> Identifier
        const name = parser_1.AncestryUtils.nth(state.activeNode.ancestry, state.ancestryIndex - 2);
        // Name hasn't been parsed yet so we can exit.
        if (!name ||
            !parser_1.XorNodeUtils.isAstChecked(name, [language_1.Ast.NodeKind.IdentifierPairedExpression])) {
            return Promise.resolve(undefined);
        }
        if (language_1.Keyword.KeywordKind.Shared.startsWith(name.node.key.literal)) {
            return Promise.resolve([language_1.Keyword.KeywordKind.Shared]);
        }
        return Promise.resolve(undefined);
    }
    // `section foo; bar = 1 |` would be expecting a semicolon.
    // The autocomplete should be for the IdentifierPairedExpression found on the previous child index.
    else if (childAttributeIndex === 3 && parser_1.XorNodeUtils.isContext(state.child)) {
        const identifierPairedExpression = parser_1.NodeIdMapUtils.assertNthChildAstChecked(state.nodeIdMapCollection, state.parent.node.id, 2, language_1.Ast.NodeKind.IdentifierPairedExpression);
        return (0, common_1.autocompleteKeywordRightMostLeaf)(state, identifierPairedExpression.id);
    }
    else {
        return Promise.resolve(undefined);
    }
}
exports.autocompleteKeywordSectionMember = autocompleteKeywordSectionMember;
//# sourceMappingURL=autocompleteKeywordSectionMember.js.map