"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocompleteKeywordRightMostLeaf = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const autocompleteKeyword_1 = require("./autocompleteKeyword");
function autocompleteKeywordRightMostLeaf(state, xorNodeId) {
    return __awaiter(this, void 0, void 0, function* () {
        // Grab the right-most Ast node in the last value.
        const rightMostAstLeafForLastValue = yield parser_1.NodeIdMapUtils.rightMostLeaf(state.nodeIdMapCollection, xorNodeId, undefined);
        if (rightMostAstLeafForLastValue === undefined) {
            return undefined;
        }
        // Start a new autocomplete inspection where the ActiveNode's ancestry is the right-most Ast node in the last value.
        const shiftedAncestry = parser_1.AncestryUtils.assertAncestry(state.nodeIdMapCollection, rightMostAstLeafForLastValue.id);
        powerquery_parser_1.Assert.isTrue(shiftedAncestry.length >= 2, "shiftedAncestry.length >= 2");
        const shiftedActiveNode = Object.assign(Object.assign({}, state.activeNode), { ancestry: shiftedAncestry });
        const inspected = yield (0, autocompleteKeyword_1.autocompleteKeyword)(state.nodeIdMapCollection, shiftedActiveNode, state.trailingToken);
        return inspected.map((autocompleteItem) => {
            if (!language_1.KeywordUtils.isKeyword(autocompleteItem.label)) {
                throw new powerquery_parser_1.CommonError.InvariantError(`expected ${autocompleteKeyword_1.autocompleteKeyword.name} to return only with KeywordKind values for AutocompleteItem.label`);
            }
            return autocompleteItem.label;
        });
    });
}
exports.autocompleteKeywordRightMostLeaf = autocompleteKeywordRightMostLeaf;
//# sourceMappingURL=common.js.map