"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryAutocompletePrimitiveType = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const constant_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language/constant/constant");
const activeNode_1 = require("../activeNode");
const __1 = require("../..");
function tryAutocompletePrimitiveType(settings, nodeIdMapCollection, activeNode, trailingToken) {
    const trace = settings.traceManager.entry(__1.AutocompleteTraceConstant.AutocompletePrimitiveType, tryAutocompletePrimitiveType.name, settings.initialCorrelationId);
    let result;
    if (!activeNode_1.ActiveNodeUtils.isPositionInBounds(activeNode)) {
        result = powerquery_parser_1.ResultUtils.ok([]);
    }
    else {
        result = powerquery_parser_1.ResultUtils.ensureResult(() => autocompletePrimitiveType(nodeIdMapCollection, activeNode, trailingToken), settings.locale);
    }
    trace.exit();
    return result;
}
exports.tryAutocompletePrimitiveType = tryAutocompletePrimitiveType;
const AllowedPrimitiveTypeConstants = constant_1.PrimitiveTypeConstants.filter((constant) => constant !== constant_1.PrimitiveTypeConstant.None);
function autocompletePrimitiveType(nodeIdMapCollection, activeNode, trailingToken) {
    if (activeNode.ancestry.length === 0) {
        return [];
    }
    const ancestry = activeNode.ancestry;
    const child = parser_1.AncestryUtils.assertNth(ancestry, 0);
    const parent = parser_1.AncestryUtils.nth(activeNode.ancestry, 1);
    if (parser_1.XorNodeUtils.isNodeKind(child, language_1.Ast.NodeKind.LiteralExpression)) {
        if (!parent) {
            return [];
        }
        else if (parser_1.XorNodeUtils.isNodeKind(parent, language_1.Ast.NodeKind.FieldTypeSpecification)) {
            return inspectFieldTypeSpecification(nodeIdMapCollection, parent, activeNode, trailingToken);
        }
        else if (parser_1.XorNodeUtils.isNodeKind(parent, language_1.Ast.NodeKind.ListType)) {
            return inspectTType(child, activeNode, trailingToken);
        }
        else if (parser_1.XorNodeUtils.isNodeKind(parent, language_1.Ast.NodeKind.NullableType)) {
            return inspectNullableType(nodeIdMapCollection, parent, activeNode, trailingToken);
        }
        else {
            return [];
        }
    }
    else if (parser_1.XorNodeUtils.isNodeKind(child, language_1.Ast.NodeKind.PrimitiveType)) {
        if (!parent) {
            return [];
        }
        else if (parser_1.XorNodeUtils.isNodeKind(parent, [
            language_1.Ast.NodeKind.AsExpression,
            language_1.Ast.NodeKind.IsExpression,
        ])) {
            return inspectEitherAsExpressionOrIsExpression(nodeIdMapCollection, parent, child, activeNode, trailingToken);
        }
        else if (parser_1.XorNodeUtils.isNodeKind(parent, language_1.Ast.NodeKind.AsNullablePrimitiveType)) {
            return inspectAsNullablePrimitiveType(nodeIdMapCollection, parent, child, activeNode, trailingToken);
        }
        else if (parser_1.XorNodeUtils.isNodeKind(parent, language_1.Ast.NodeKind.NullablePrimitiveType)) {
            return inspectNullablePrimitiveType(nodeIdMapCollection, parent, child, activeNode, trailingToken);
        }
        else {
            return inspectPrimitiveType(child, activeNode, trailingToken);
        }
    }
    else if (parser_1.XorNodeUtils.isNodeKind(child, language_1.Ast.NodeKind.TypePrimaryType)) {
        return inspectTypePrimaryType(nodeIdMapCollection, child, activeNode, trailingToken);
    }
    else {
        return [];
    }
}
// Defaults to insertion if range is undefined,
// otherwise replaces the range with the label.
function createAutocompleteItems(text, range) {
    return AllowedPrimitiveTypeConstants.map((label) => {
        const jaroWinklerScore = text !== undefined ? (0, __1.calculateJaroWinkler)(label, text) : 1;
        return {
            jaroWinklerScore,
            kind: __1.CompletionItemKind.Keyword,
            label,
            powerQueryType: language_1.TypeUtils.primitiveType(label === language_1.Constant.PrimitiveTypeConstant.Null, language_1.TypeUtils.typeKindFromPrimitiveTypeConstantKind(label)),
            textEdit: range ? __1.TextEdit.replace(range, label) : undefined,
        };
    });
}
function createAutocompleteItemsFromIdentifierExpression(identifierExpression, activeNode) {
    if (identifierExpression.inclusiveConstant) {
        return [];
    }
    const literal = identifierExpression.identifier.literal;
    if (!isSubstringOfPrimitiveTypeConstant(literal) ||
        !__1.PositionUtils.isInAst(activeNode.position, identifierExpression, true, true)) {
        return [];
    }
    return createAutocompleteItems(literal, __1.Range.create(__1.PositionUtils.positionFromTokenPosition(identifierExpression.tokenRange.positionStart), __1.PositionUtils.positionFromTokenPosition(identifierExpression.tokenRange.positionEnd)));
}
// `any` returns ["any", "anynonnull"]
// `date` returns ["date", "datetime", "datetimezone"]
// etc.
function createAutocompleteItemsFromPrimitiveTypeConstant(primitiveType, activeNode) {
    if (!__1.PositionUtils.isInAst(activeNode.position, primitiveType, true, true)) {
        return [];
    }
    return createAutocompleteItems(primitiveType.primitiveTypeKind, __1.Range.create(__1.PositionUtils.positionFromTokenPosition(primitiveType.tokenRange.positionStart), __1.PositionUtils.positionFromTokenPosition(primitiveType.tokenRange.positionEnd)));
}
// Returns AllowedPrimitiveTypeConstants if either:
//  - there is no trailing token
//  - the Position is either on or to the left of the trailing token's start
// elif Position is not on the trailing token's end, then return an empty array
// else return the autocomplete items for the trailing text.
function createAutocompleteItemsFromTrailingToken(trailingToken) {
    if (!trailingToken) {
        return createAutocompleteItems();
    }
    else if (!trailingToken.isPositionEitherInOrOnToken) {
        return [];
    }
    else {
        return createAutocompleteItems(trailingToken.data, __1.Range.create(__1.PositionUtils.positionFromTokenPosition(trailingToken.positionStart), __1.PositionUtils.positionFromTokenPosition(trailingToken.positionEnd)));
    }
}
function inspectAsNullablePrimitiveType(nodeIdMapCollection, asNullablePrimitiveType, primitiveType, activeNode, trailingToken) {
    // Ast
    if (parser_1.XorNodeUtils.isAst(asNullablePrimitiveType)) {
        const nullablePrimitiveType = asNullablePrimitiveType.node.paired;
        switch (nullablePrimitiveType.kind) {
            case language_1.Ast.NodeKind.NullablePrimitiveType:
                return inspectNullablePrimitiveType(nodeIdMapCollection, parser_1.XorNodeUtils.boxAst(nullablePrimitiveType), parser_1.XorNodeUtils.boxAst(nullablePrimitiveType.paired), activeNode, trailingToken);
            case language_1.Ast.NodeKind.PrimitiveType:
                return inspectPrimitiveType(parser_1.XorNodeUtils.boxAst(nullablePrimitiveType), activeNode, trailingToken);
            default:
                throw powerquery_parser_1.Assert.isNever(nullablePrimitiveType);
        }
    }
    // ParseContext
    else {
        const asConstant = parser_1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, asNullablePrimitiveType.node.id, 0, [language_1.Ast.NodeKind.Constant]);
        if (asConstant === undefined || !__1.PositionUtils.isAfterAst(activeNode.position, asConstant, true)) {
            return [];
        }
        const nullablePrimitiveType = parser_1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, asNullablePrimitiveType.node.id, 1, [language_1.Ast.NodeKind.NullablePrimitiveType, language_1.Ast.NodeKind.PrimitiveType]);
        if (nullablePrimitiveType === undefined) {
            return [];
        }
        else if (parser_1.XorNodeUtils.isNodeKind(nullablePrimitiveType, language_1.Ast.NodeKind.NullablePrimitiveType)) {
            return inspectNullablePrimitiveType(nodeIdMapCollection, nullablePrimitiveType, primitiveType, activeNode, trailingToken);
        }
        else if (parser_1.XorNodeUtils.isNodeKind(nullablePrimitiveType, language_1.Ast.NodeKind.PrimitiveType)) {
            return inspectPrimitiveType(nullablePrimitiveType, activeNode, trailingToken);
        }
        else {
            throw new powerquery_parser_1.CommonError.InvariantError(`expected either a NullablePrimitiveType or PrimitiveType`, {
                nodeKind: nullablePrimitiveType.node.kind,
            });
        }
    }
}
function inspectEitherAsExpressionOrIsExpression(nodeIdMapCollection, eitherAsExpressionOrIsExpression, primitiveType, activeNode, trailingToken) {
    // Ast
    if (parser_1.XorNodeUtils.isAst(primitiveType)) {
        return createAutocompleteItemsFromPrimitiveTypeConstant(primitiveType.node, activeNode);
    }
    // ParseContext
    else {
        const eitherAsConstantOrIsConstant = parser_1.NodeIdMapUtils.assertNthChildAstChecked(nodeIdMapCollection, eitherAsExpressionOrIsExpression.node.id, 1, language_1.Ast.NodeKind.Constant);
        // `1 is|`
        if (__1.PositionUtils.isOnAstEnd(activeNode.position, eitherAsConstantOrIsConstant)) {
            return [];
        }
        return createAutocompleteItemsFromTrailingToken(trailingToken);
    }
}
function inspectFieldTypeSpecification(nodeIdMapCollection, fieldTypeSpecification, activeNode, trailingToken) {
    // Ast
    if (parser_1.XorNodeUtils.isAst(fieldTypeSpecification)) {
        return inspectTType(parser_1.XorNodeUtils.boxAst(fieldTypeSpecification.node.fieldType), activeNode, trailingToken);
    }
    // ParseContext
    else {
        const equalConstant = parser_1.NodeIdMapUtils.assertNthChildAstChecked(nodeIdMapCollection, fieldTypeSpecification.node.id, 0, language_1.Ast.NodeKind.Constant);
        if (equalConstant === undefined || !__1.PositionUtils.isAfterAst(activeNode.position, equalConstant, false)) {
            return [];
        }
        const ttype = parser_1.NodeIdMapUtils.nthChildXor(nodeIdMapCollection, fieldTypeSpecification.node.id, 1);
        if (ttype === undefined) {
            return createAutocompleteItems();
        }
        else if (parser_1.XorNodeUtils.isTType(ttype)) {
            return inspectTType(ttype, activeNode, trailingToken);
        }
    }
    return [];
}
function inspectNullablePrimitiveType(nodeIdMapCollection, nullablePrimitiveType, primitiveType, activeNode, trailingToken) {
    // Ast
    if (parser_1.XorNodeUtils.isAst(primitiveType)) {
        return inspectPrimitiveType(primitiveType, activeNode, undefined);
    }
    // ParseContext
    else if (!trailingToken) {
        return createAutocompleteItems();
    }
    else if (!trailingToken.isPositionEitherInOrOnToken) {
        return [];
    }
    else {
        const nullableConstant = parser_1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, nullablePrimitiveType.node.id, 0, [language_1.Ast.NodeKind.Constant]);
        if (nullableConstant === undefined || !__1.PositionUtils.isAfterAst(activeNode.position, nullableConstant, true)) {
            return [];
        }
        else {
            return inspectPrimitiveType(primitiveType, activeNode, trailingToken);
        }
    }
}
function inspectNullableType(nodeIdMapCollection, nullableType, activeNode, trailingToken) {
    // Ast
    if (parser_1.XorNodeUtils.isAst(nullableType)) {
        // `type nullable date|`
        if (language_1.AstUtils.isNodeKind(nullableType.node.paired, language_1.Ast.NodeKind.PrimitiveType)) {
            return inspectPrimitiveType(parser_1.XorNodeUtils.boxAst(nullableType.node.paired), activeNode, trailingToken);
        }
        // `type nullable a|`
        else if (language_1.AstUtils.isNodeKind(nullableType.node.paired, language_1.Ast.NodeKind.IdentifierExpression)) {
            return createAutocompleteItemsFromIdentifierExpression(nullableType.node.paired, activeNode);
        }
        else {
            return [];
        }
    }
    // ParseContext
    else {
        const nullableConstant = parser_1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, nullableType.node.id, 0, [language_1.Ast.NodeKind.Constant]);
        if (nullableConstant === undefined || !__1.PositionUtils.isAfterAst(activeNode.position, nullableConstant, true)) {
            return [];
        }
        else {
            const primitiveType = parser_1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, nullableType.node.id, 1, [language_1.Ast.NodeKind.LiteralExpression, language_1.Ast.NodeKind.PrimitiveType]);
            if (primitiveType === undefined) {
                return [];
            }
            // The parser defaults to having thrown on a LiteralExpression if nothing trails afterwords,
            // eg. `type nullable |`
            else if (parser_1.XorNodeUtils.isNodeKind(primitiveType, language_1.Ast.NodeKind.LiteralExpression)) {
                return createAutocompleteItems();
            }
            else if (parser_1.XorNodeUtils.isNodeKind(primitiveType, language_1.Ast.NodeKind.PrimitiveType)) {
                return inspectPrimitiveType(primitiveType, activeNode, trailingToken);
            }
            else {
                return [];
            }
        }
    }
}
function inspectPrimitiveType(primitiveTypeXor, activeNode, trailingToken) {
    // Ast
    if (parser_1.XorNodeUtils.isAst(primitiveTypeXor)) {
        if (!__1.PositionUtils.isInAst(activeNode.position, primitiveTypeXor.node, true, true)) {
            return [];
        }
        const primitiveTypeAst = primitiveTypeXor.node;
        const primitiveTypeKind = primitiveTypeAst.primitiveTypeKind;
        return createAutocompleteItems(primitiveTypeKind, __1.Range.create(__1.PositionUtils.positionFromTokenPosition(primitiveTypeAst.tokenRange.positionStart), __1.PositionUtils.positionFromTokenPosition(primitiveTypeAst.tokenRange.positionEnd)));
    }
    // ParseContext
    else if (trailingToken) {
        const trailingText = trailingToken.data;
        if (isSubstringOfPrimitiveTypeConstant(trailingText)) {
            return createAutocompleteItems(trailingText, __1.Range.create(__1.PositionUtils.positionFromTokenPosition(trailingToken.positionStart), __1.PositionUtils.positionFromTokenPosition(trailingToken.positionEnd)));
        }
    }
    return createAutocompleteItems();
}
function inspectTType(ttype, activeNode, trailingToken) {
    // Ast
    if (parser_1.XorNodeUtils.isAst(ttype)) {
        // `type nullable date|`
        if (language_1.AstUtils.isNodeKind(ttype.node, language_1.Ast.NodeKind.PrimitiveType)) {
            return inspectPrimitiveType(parser_1.XorNodeUtils.boxAst(ttype.node), activeNode, trailingToken);
        }
        // `type nullable a|`
        else if (language_1.AstUtils.isNodeKind(ttype.node, language_1.Ast.NodeKind.IdentifierExpression)) {
            return createAutocompleteItemsFromIdentifierExpression(ttype.node, activeNode);
        }
    }
    // ParseContext
    // The parser defaults to having thrown on a LiteralExpression if nothing trails afterwords,
    // eg. `type nullable |`
    else if (parser_1.XorNodeUtils.isNodeKind(ttype, language_1.Ast.NodeKind.LiteralExpression)) {
        return createAutocompleteItems();
    }
    else if (parser_1.XorNodeUtils.isNodeKind(ttype, language_1.Ast.NodeKind.PrimitiveType)) {
        return inspectPrimitiveType(ttype, activeNode, trailingToken);
    }
    return [];
}
function inspectTypePrimaryType(nodeIdMapCollection, typePrimaryType, activeNode, trailingToken) {
    const typeConstant = parser_1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, typePrimaryType.node.id, 0, language_1.Ast.NodeKind.Constant);
    if (typeConstant === undefined ||
        !parser_1.XorNodeUtils.isAst(typeConstant) ||
        !__1.PositionUtils.isAfterAst(activeNode.position, typeConstant.node, true)) {
        return [];
    }
    const primaryType = parser_1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, typePrimaryType.node.id, 1, [
        language_1.Ast.NodeKind.PrimitiveType,
        language_1.Ast.NodeKind.FunctionType,
        language_1.Ast.NodeKind.ListType,
        language_1.Ast.NodeKind.NullableType,
        language_1.Ast.NodeKind.RecordType,
        language_1.Ast.NodeKind.TableType,
    ]);
    if (primaryType === undefined) {
        return createAutocompleteItemsFromTrailingToken(trailingToken);
    }
    // Ast
    else if (parser_1.XorNodeUtils.isAst(primaryType)) {
        if (!__1.PositionUtils.isOnAstEnd(activeNode.position, primaryType.node) ||
            !language_1.AstUtils.isNodeKind(primaryType.node, language_1.Ast.NodeKind.PrimitiveType)) {
            return [];
        }
        return createAutocompleteItemsFromPrimitiveTypeConstant(primaryType.node, activeNode);
    }
    else {
        return [];
    }
}
function isSubstringOfPrimitiveTypeConstant(text) {
    return AllowedPrimitiveTypeConstants.some((value) => value.includes(text));
}
//# sourceMappingURL=autocompletePrimitiveType.js.map