"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocomplete = void 0;
const PQP = require("@microsoft/powerquery-parser");
const __1 = require("../..");
const trailingTokenUtils_1 = require("./trailingTokenUtils");
const autocompleteFieldAccess_1 = require("./autocompleteFieldAccess");
const autocompleteKeyword_1 = require("./autocompleteKeyword/autocompleteKeyword");
const autocompleteLanguageConstant_1 = require("./autocompleteLanguageConstant");
const autocompletePrimitiveType_1 = require("./autocompletePrimitiveType");
// Given some Position and ParseState, return autocomplete suggestions.
function autocomplete(settings, parseState, typeCache, activeNode, parseError) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry(__1.AutocompleteTraceConstant.Autocomplete, autocomplete.name, settings.initialCorrelationId);
        const updatedSettings = Object.assign(Object.assign({}, settings), { initialCorrelationId: trace.id });
        const nodeIdMapCollection = parseState.contextState.nodeIdMapCollection;
        let trailingToken;
        if (parseError !== undefined) {
            const parseErrorToken = PQP.Parser.ParseError.tokenFrom(parseError.innerError);
            if (parseErrorToken !== undefined) {
                trailingToken = (0, trailingTokenUtils_1.createTrailingToken)(activeNode.position, parseErrorToken);
            }
        }
        trace.exit();
        return {
            triedFieldAccess: yield (0, autocompleteFieldAccess_1.tryAutocompleteFieldAccess)(updatedSettings, parseState, activeNode, typeCache),
            triedKeyword: yield (0, autocompleteKeyword_1.tryAutocompleteKeyword)(updatedSettings, nodeIdMapCollection, activeNode, trailingToken),
            triedLanguageConstant: (0, autocompleteLanguageConstant_1.tryAutocompleteLanguageConstant)(updatedSettings, nodeIdMapCollection, activeNode, trailingToken),
            triedPrimitiveType: (0, autocompletePrimitiveType_1.tryAutocompletePrimitiveType)(updatedSettings, nodeIdMapCollection, activeNode, trailingToken),
        };
    });
}
exports.autocomplete = autocomplete;
//# sourceMappingURL=task.js.map