"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryDeferenceIdentifier = void 0;
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const scope_1 = require("./scope");
const __1 = require("..");
const trace_2 = require("../trace");
// Recusrive deference of the identifier until it reaches the value node.
// Does not handle recursive identifiers.
function tryDeferenceIdentifier(settings, nodeIdMapCollection, eachScopeById, xorNode, 
// If a map is given, then it's mutated and returned.
// Else create a new Map instance and return that instead.
scopeById = new Map()) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry(trace_2.InspectionTraceConstant.InspectScope, tryDeferenceIdentifier.name, settings.initialCorrelationId, __1.TraceUtils.xorNodeDetails(xorNode));
        const updatedSettings = Object.assign(Object.assign({}, settings), { initialCorrelationId: trace.id });
        parser_1.XorNodeUtils.assertIsIdentifier(xorNode);
        if (parser_1.XorNodeUtils.isContext(xorNode)) {
            trace.exit({ [trace_1.TraceConstant.Result]: undefined });
            return powerquery_parser_1.ResultUtils.ok(undefined);
        }
        const identifier = xorNode.node;
        let identifierLiteral;
        switch (identifier.kind) {
            case language_1.Ast.NodeKind.Identifier:
                identifierLiteral = identifier.literal;
                break;
            case language_1.Ast.NodeKind.IdentifierExpression:
                identifierLiteral = identifier.identifier.literal;
                break;
            default:
                throw powerquery_parser_1.Assert.isNever(identifier);
        }
        const triedNodeScope = yield (0, scope_1.assertGetOrCreateNodeScope)(updatedSettings, nodeIdMapCollection, eachScopeById, xorNode.node.id, scopeById);
        if (powerquery_parser_1.ResultUtils.isError(triedNodeScope)) {
            trace.exit({ [trace_1.TraceConstant.Result]: triedNodeScope.kind });
            return triedNodeScope;
        }
        const nodeScope = triedNodeScope.value;
        const scopeItem = nodeScope.get(identifierLiteral);
        if (
        // If the identifier couldn't be found in the generated scope,
        // then either the scope generation is incorrect or it's an external identifier.
        scopeItem === undefined) {
            trace.exit({ [trace_1.TraceConstant.Result]: undefined });
            return powerquery_parser_1.ResultUtils.ok(xorNode);
        }
        let nextXorNode;
        switch (scopeItem.kind) {
            case scope_1.ScopeItemKind.Each:
            case scope_1.ScopeItemKind.Parameter:
            case scope_1.ScopeItemKind.Undefined:
                break;
            case scope_1.ScopeItemKind.LetVariable:
            case scope_1.ScopeItemKind.RecordField:
                nextXorNode = scopeItem.value;
                break;
            case scope_1.ScopeItemKind.SectionMember:
                nextXorNode = scopeItem.value;
                break;
            default:
                throw powerquery_parser_1.Assert.isNever(scopeItem);
        }
        let result;
        if (nextXorNode === undefined) {
            result = Promise.resolve(powerquery_parser_1.ResultUtils.ok(xorNode));
        }
        else if (parser_1.XorNodeUtils.isContext(nextXorNode) ||
            (nextXorNode.node.kind !== language_1.Ast.NodeKind.Identifier &&
                nextXorNode.node.kind !== language_1.Ast.NodeKind.IdentifierExpression)) {
            result = Promise.resolve(powerquery_parser_1.ResultUtils.ok(xorNode));
        }
        else {
            result = tryDeferenceIdentifier(updatedSettings, nodeIdMapCollection, eachScopeById, nextXorNode, scopeById);
        }
        trace.exit();
        return result;
    });
}
exports.tryDeferenceIdentifier = tryDeferenceIdentifier;
//# sourceMappingURL=deferenceIdentifier.js.map