"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryCurrentInvokeExpression = void 0;
const PQP = require("@microsoft/powerquery-parser");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const activeNode_1 = require("../activeNode");
const invokeExpression_1 = require("./invokeExpression");
const typeCache_1 = require("../typeCache");
const __1 = require("../..");
// If the cursor is in an InvokeExpression then return:
//  * the number of given arguments
//  * the number of expected arguments
//  * the given argument nodes
//  * the given argument types
//  * type checking for parameter types vs given types
function tryCurrentInvokeExpression(settings, nodeIdMapCollection, activeNode, 
// If a TypeCache is given, then potentially add to its values and include it as part of the return,
// Else create a new TypeCache and include it in the return.
typeCache = typeCache_1.TypeCacheUtils.emptyCache()) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry(__1.InspectionTraceConstant.InspectCurrentInvokeExpression, tryCurrentInvokeExpression.name, settings.initialCorrelationId);
        const updatedSettings = Object.assign(Object.assign({}, settings), { initialCorrelationId: trace.id });
        if (!activeNode_1.ActiveNodeUtils.isPositionInBounds(activeNode)) {
            trace.exit();
            return Promise.resolve(powerquery_parser_1.ResultUtils.ok(undefined));
        }
        const result = yield powerquery_parser_1.ResultUtils.ensureResultAsync(() => inspectInvokeExpression(updatedSettings, nodeIdMapCollection, activeNode, typeCache, trace.id), updatedSettings.locale);
        trace.exit();
        return result;
    });
}
exports.tryCurrentInvokeExpression = tryCurrentInvokeExpression;
function inspectInvokeExpression(settings, nodeIdMapCollection, activeNode, typeCache, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry(__1.InspectionTraceConstant.InspectCurrentInvokeExpression, inspectInvokeExpression.name, correlationId);
        const updatedSettings = Object.assign(Object.assign({}, settings), { initialCorrelationId: trace.id });
        const ancestry = activeNode.ancestry;
        const ancestryIndex = parser_1.AncestryUtils.indexOfNodeKind(ancestry, language_1.Ast.NodeKind.InvokeExpression);
        if (ancestryIndex === undefined) {
            trace.exit();
            return undefined;
        }
        const invokeExpressionXorNode = parser_1.AncestryUtils.assertNth(ancestry, ancestryIndex);
        const triedInvokeExpression = yield (0, invokeExpression_1.tryInvokeExpression)(updatedSettings, nodeIdMapCollection, invokeExpressionXorNode.node.id, typeCache);
        if (powerquery_parser_1.ResultUtils.isError(triedInvokeExpression)) {
            trace.exit({ [trace_1.TraceConstant.IsThrowing]: true });
            throw triedInvokeExpression;
        }
        const invokeExpression = triedInvokeExpression.value;
        const args = invokeExpression.arguments;
        let argumentOrdinal;
        if (args !== undefined) {
            argumentOrdinal = Object.assign(Object.assign({}, args), { argumentOrdinal: getArgumentOrdinal(nodeIdMapCollection, activeNode, ancestryIndex, invokeExpressionXorNode) });
        }
        const result = Object.assign(Object.assign({}, invokeExpression), { arguments: argumentOrdinal });
        trace.exit();
        return result;
    });
}
function getArgumentOrdinal(nodeIdMapCollection, activeNode, ancestryIndex, invokeExpressionXorNode) {
    var _a;
    // `foo(1|)
    const ancestryCsv = parser_1.AncestryUtils.nthChecked(activeNode.ancestry, ancestryIndex - 2, language_1.Ast.NodeKind.Csv);
    if (ancestryCsv !== undefined) {
        return (_a = ancestryCsv.node.attributeIndex) !== null && _a !== void 0 ? _a : 0;
    }
    const previousXor = parser_1.AncestryUtils.nthChecked(activeNode.ancestry, ancestryIndex - 1, [
        // `foo(|)`
        language_1.Ast.NodeKind.ArrayWrapper,
        // `foo(1|`
        language_1.Ast.NodeKind.Constant,
    ]);
    let arrayWrapperXorNode;
    switch (previousXor === null || previousXor === void 0 ? void 0 : previousXor.node.kind) {
        case language_1.Ast.NodeKind.Constant: {
            arrayWrapperXorNode = parser_1.NodeIdMapUtils.assertNthChildXorChecked(nodeIdMapCollection, invokeExpressionXorNode.node.id, 1, language_1.Ast.NodeKind.ArrayWrapper);
            break;
        }
        case language_1.Ast.NodeKind.ArrayWrapper: {
            arrayWrapperXorNode = previousXor;
            break;
        }
        case undefined:
        default:
            throw new PQP.CommonError.InvariantError(`encountered an unknown scenario for getArgumentOrdinal`, {
                ancestryIndex,
                ancestryNodeKinds: activeNode.ancestry.map((xorNode) => xorNode.node.kind),
            });
    }
    const arrayWrapperChildIds = nodeIdMapCollection.childIdsById.get(arrayWrapperXorNode.node.id);
    return arrayWrapperChildIds !== undefined ? arrayWrapperChildIds.length - 1 : 0;
}
//# sourceMappingURL=currentInvokeExpression.js.map