"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.pseudoFunctionExpressionType = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
function pseudoFunctionExpressionType(nodeIdMapCollection, fnExpr) {
    parser_1.XorNodeUtils.assertIsNodeKind(fnExpr, language_1.Ast.NodeKind.FunctionExpression);
    const examinedParameters = [];
    // Iterates all parameters as TXorNodes if they exist, otherwise early exists from an empty list.
    for (const parameter of functionParameterXorNodes(nodeIdMapCollection, fnExpr)) {
        // A parameter isn't examinable if it doesn't have an Ast.Identifier for its name.
        const name = parser_1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, parameter.node.id, 1, language_1.Ast.NodeKind.Identifier);
        if (name === undefined) {
            break;
        }
        const examinable = language_1.TypeUtils.inspectParameter(nodeIdMapCollection, parser_1.XorNodeUtils.assertAsNodeKind(parameter, language_1.Ast.NodeKind.Parameter));
        if (examinable !== undefined) {
            examinedParameters.push({
                id: parameter.node.id,
                isNullable: examinable.isNullable,
                isOptional: examinable.isOptional,
                type: examinable.type,
                name,
            });
        }
    }
    const returnType = parser_1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, fnExpr.node.id, 1, language_1.Ast.NodeKind.AsNullablePrimitiveType);
    let isReturnNullable;
    let returnTypeKind;
    if (returnType !== undefined) {
        const simplified = language_1.AstUtils.simplifyAsNullablePrimitiveType(returnType);
        isReturnNullable = simplified.isNullable;
        returnTypeKind = language_1.TypeUtils.typeKindFromPrimitiveTypeConstantKind(simplified.primitiveTypeConstantKind);
    }
    else {
        isReturnNullable = true;
        returnTypeKind = language_1.Type.TypeKind.Any;
    }
    return {
        parameters: examinedParameters,
        returnType: {
            kind: returnTypeKind,
            extendedKind: undefined,
            isNullable: isReturnNullable,
        },
    };
}
exports.pseudoFunctionExpressionType = pseudoFunctionExpressionType;
function functionParameterXorNodes(nodeIdMapCollection, fnExpr) {
    const parametersList = parser_1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, fnExpr.node.id, 0, language_1.Ast.NodeKind.ParameterList);
    if (parametersList === undefined) {
        return [];
    }
    const wrappedContent = parser_1.NodeIdMapUtils.wrappedContentXor(nodeIdMapCollection, parametersList.node.id);
    return wrappedContent === undefined ? [] : parser_1.NodeIdMapIterator.iterArrayWrapper(nodeIdMapCollection, wrappedContent);
}
//# sourceMappingURL=pseudoFunctionExpressionType.js.map