import * as PQP from "@microsoft/powerquery-parser";
import { Ast, Constant, Type } from "@microsoft/powerquery-parser/lib/powerquery-parser/language";
import { TXorNode } from "@microsoft/powerquery-parser/lib/powerquery-parser/parser";
export declare type ScopeTypeByKey = Map<string, Type.TPowerQueryType>;
export declare type TriedNodeScope = PQP.Result<NodeScope, PQP.CommonError.CommonError>;
export declare type ScopeById = Map<number, NodeScope>;
export declare type NodeScope = Map<string, TScopeItem>;
export declare type TScopeItem = EachScopeItem | LetVariableScopeItem | ParameterScopeItem | RecordFieldScopeItem | SectionMemberScopeItem | UndefinedScopeItem;
export declare type TKeyValuePairScopeItem = LetVariableScopeItem | RecordFieldScopeItem | SectionMemberScopeItem;
export declare enum ScopeItemKind {
    Each = "Each",
    LetVariable = "LetVariable",
    Parameter = "Parameter",
    RecordField = "RecordField",
    SectionMember = "SectionMember",
    Undefined = "Undefined"
}
export interface IScopeItem {
    readonly kind: ScopeItemKind;
    readonly id: number;
    readonly isRecursive: boolean;
}
export interface IKeyValuePairScopeItem<Key extends Ast.Identifier | Ast.GeneralizedIdentifier, Kind extends ScopeItemKind.LetVariable | ScopeItemKind.RecordField | ScopeItemKind.SectionMember> extends IScopeItem {
    readonly kind: Kind;
    readonly key: Key;
    readonly value: TXorNode | undefined;
}
export interface EachScopeItem extends IScopeItem {
    readonly kind: ScopeItemKind.Each;
    readonly eachExpression: TXorNode;
    readonly implicitParameterType: Type.TPowerQueryType;
}
export declare type LetVariableScopeItem = IKeyValuePairScopeItem<Ast.Identifier, ScopeItemKind.LetVariable>;
export interface ParameterScopeItem extends IScopeItem {
    readonly kind: ScopeItemKind.Parameter;
    readonly name: Ast.Identifier;
    readonly isOptional: boolean;
    readonly isNullable: boolean;
    readonly type: Constant.PrimitiveTypeConstant | undefined;
}
export declare type RecordFieldScopeItem = IKeyValuePairScopeItem<Ast.GeneralizedIdentifier, ScopeItemKind.RecordField>;
export declare type SectionMemberScopeItem = IKeyValuePairScopeItem<Ast.Identifier, ScopeItemKind.SectionMember>;
export interface UndefinedScopeItem extends IScopeItem {
    readonly kind: ScopeItemKind.Undefined;
    readonly xorNode: TXorNode;
}
