"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryInspect = exports.inspect = exports.InspectionInstance = void 0;
const PQP = require("@microsoft/powerquery-parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const activeNode_1 = require("./activeNode");
const scopeUtils_1 = require("./scope/scopeUtils");
const __1 = require("..");
const expectedType_1 = require("./expectedType");
const scope_1 = require("./scope");
const type_1 = require("./type");
const typeCache_1 = require("./typeCache");
const autocomplete_1 = require("./autocomplete");
const currentInvokeExpression_1 = require("./invokeExpression/currentInvokeExpression");
class InspectionInstance {
    constructor(settings, nodeIdMapCollection, activeNode, autocomplete, triedCurrentInvokeExpression, triedNodeScope, triedScopeType, triedExpectedType, typeCache, parseState) {
        this.settings = settings;
        this.nodeIdMapCollection = nodeIdMapCollection;
        this.activeNode = activeNode;
        this.autocomplete = autocomplete;
        this.triedCurrentInvokeExpression = triedCurrentInvokeExpression;
        this.triedNodeScope = triedNodeScope;
        this.triedScopeType = triedScopeType;
        this.triedExpectedType = triedExpectedType;
        this.typeCache = typeCache;
        this.parseState = parseState;
    }
    tryNodeScope(id) {
        const trace = this.settings.traceManager.entry(__1.InspectionTraceConstant.Inspect, scope_1.tryNodeScope.name, this.settings.initialCorrelationId);
        const updatedSettings = Object.assign(Object.assign({}, this.settings), { initialCorrelationId: trace.id });
        const result = (0, scope_1.tryNodeScope)(updatedSettings, this.nodeIdMapCollection, this.settings.eachScopeById, id, this.typeCache.scopeById);
        trace.exit();
        return result;
    }
    collectAllIdentifiersBeneath(valueCreator) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function* () {
            const astNodeById = this.nodeIdMapCollection.astNodeById;
            const childIdsById = this.nodeIdMapCollection.childIdsById;
            const originLiteral = valueCreator.literal;
            const entry = (0, scopeUtils_1.findDirectUpperScopeExpression)(this.nodeIdMapCollection, valueCreator.id);
            if (entry) {
                const allIdentifierIdSet = (_a = this.nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.Identifier)) !== null && _a !== void 0 ? _a : new Set();
                const allGeneralizedIdentifierIdSet = (_b = this.nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.GeneralizedIdentifier)) !== null && _b !== void 0 ? _b : new Set();
                const current = entry;
                const idsByTiers = [];
                let currentTier = ((_c = childIdsById.get(current.id)) !== null && _c !== void 0 ? _c : []).slice();
                while (currentTier.length) {
                    const toBePushed = currentTier.filter((one) => allIdentifierIdSet.has(one) || allGeneralizedIdentifierIdSet.has(one));
                    toBePushed.length && idsByTiers.push(toBePushed);
                    currentTier = currentTier.slice();
                    let nextTier = [];
                    while (currentTier.length) {
                        const oneNode = (_d = currentTier.shift()) !== null && _d !== void 0 ? _d : -1;
                        const childrenOfTheNode = ((_e = childIdsById.get(oneNode)) !== null && _e !== void 0 ? _e : []).slice();
                        nextTier = nextTier.concat(childrenOfTheNode);
                    }
                    currentTier = nextTier;
                }
                // filter literal names
                const completeIdentifierNodes = idsByTiers
                    .flat(1)
                    .map((one) => astNodeById.get(one))
                    .filter(Boolean);
                let filteredIdentifierNodes = completeIdentifierNodes.filter((one) => one.literal === originLiteral);
                // populate the scope items for each
                yield Promise.all(filteredIdentifierNodes
                    .filter((oneIdentifier) => oneIdentifier.kind === language_1.Ast.NodeKind.GeneralizedIdentifier ||
                    oneIdentifier.identifierContextKind === language_1.Ast.IdentifierContextKind.Value)
                    .map((oneIdentifier) => this.tryNodeScope(oneIdentifier.id)));
                filteredIdentifierNodes = filteredIdentifierNodes.filter((oneIdentifierNode) => {
                    if (oneIdentifierNode.kind === language_1.Ast.NodeKind.GeneralizedIdentifier) {
                        return oneIdentifierNode.id === valueCreator.id;
                    }
                    else if (oneIdentifierNode.identifierContextKind === language_1.Ast.IdentifierContextKind.Value) {
                        const theScope = this.typeCache.scopeById.get(oneIdentifierNode.id);
                        const theScopeItem = (0, scopeUtils_1.findScopeItemByLiteral)(theScope, originLiteral);
                        const theCreatorIdentifier = (0, scopeUtils_1.scopeCreatorIdentifier)(theScopeItem);
                        return theCreatorIdentifier && theCreatorIdentifier.id === valueCreator.id;
                    }
                    else if (oneIdentifierNode.identifierContextKind === language_1.Ast.IdentifierContextKind.Key ||
                        oneIdentifierNode.identifierContextKind === language_1.Ast.IdentifierContextKind.Parameter) {
                        return oneIdentifierNode.id === valueCreator.id;
                    }
                    return false;
                });
                return filteredIdentifierNodes;
            }
            return [];
        });
    }
}
exports.InspectionInstance = InspectionInstance;
function inspect(settings, parseState, parseError, position, 
// If a TypeCache is given, then potentially add to its values and include it as part of the return,
// Else create a new TypeCache and include it in the return.
typeCache = typeCache_1.TypeCacheUtils.emptyCache()) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry(__1.InspectionTraceConstant.Inspect, inspect.name, settings.initialCorrelationId);
        const updatedSettings = Object.assign(Object.assign({}, settings), { initialCorrelationId: trace.id });
        const nodeIdMapCollection = parseState.contextState.nodeIdMapCollection;
        // We should only get an undefined for activeNode iff the document is empty
        const activeNode = activeNode_1.ActiveNodeUtils.activeNode(nodeIdMapCollection, position);
        const triedCurrentInvokeExpression = (0, currentInvokeExpression_1.tryCurrentInvokeExpression)(updatedSettings, nodeIdMapCollection, activeNode, typeCache);
        let triedNodeScope;
        let triedScopeType;
        let triedExpectedType;
        if (activeNode_1.ActiveNodeUtils.isPositionInBounds(activeNode)) {
            triedNodeScope = (0, scope_1.tryNodeScope)(updatedSettings, nodeIdMapCollection, updatedSettings.eachScopeById, activeNode_1.ActiveNodeUtils.assertGetLeaf(activeNode).node.id, typeCache.scopeById);
            const ancestryLeaf = PQP.Parser.AncestryUtils.assertNth(activeNode.ancestry, 0);
            triedScopeType = (0, type_1.tryScopeType)(updatedSettings, nodeIdMapCollection, ancestryLeaf.node.id, typeCache);
            triedExpectedType = (0, expectedType_1.tryExpectedType)(updatedSettings, activeNode);
        }
        else {
            triedNodeScope = Promise.resolve(PQP.ResultUtils.ok(new Map()));
            triedScopeType = Promise.resolve(PQP.ResultUtils.ok(new Map()));
            triedExpectedType = PQP.ResultUtils.ok(undefined);
        }
        const result = new InspectionInstance(updatedSettings, nodeIdMapCollection, activeNode, yield (0, autocomplete_1.autocomplete)(updatedSettings, parseState, typeCache, activeNode, parseError), triedCurrentInvokeExpression, triedNodeScope, triedScopeType, triedExpectedType, typeCache, parseState);
        trace.exit();
        return result;
    });
}
exports.inspect = inspect;
function tryInspect(settings, text, position, typeCache = typeCache_1.TypeCacheUtils.emptyCache()) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry(__1.InspectionTraceConstant.Inspect, tryInspect.name, settings.initialCorrelationId);
        const updatedSettings = Object.assign(Object.assign({}, settings), { initialCorrelationId: trace.id });
        const triedLexParse = yield PQP.TaskUtils.tryLexParse(updatedSettings, text);
        let parseState;
        let parseError;
        if (PQP.TaskUtils.isLexStageError(triedLexParse) || PQP.TaskUtils.isParseStageCommonError(triedLexParse)) {
            trace.exit({ [trace_1.TraceConstant.IsError]: true });
            return PQP.ResultUtils.error(triedLexParse.error);
        }
        else if (PQP.TaskUtils.isParseStageError(triedLexParse)) {
            parseState = triedLexParse.parseState;
            parseError = triedLexParse.error;
        }
        else {
            parseState = triedLexParse.parseState;
        }
        const result = PQP.ResultUtils.ok(inspect(updatedSettings, parseState, parseError, position, typeCache));
        trace.exit({ [trace_1.TraceConstant.IsError]: false });
        return result;
    });
}
exports.tryInspect = tryInspect;
//# sourceMappingURL=task.js.map