"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createParameterType = exports.dereferencedIdentifierType = exports.inspectXor = exports.inspectTypeFromChildAttributeIndex = exports.inspectScopeItem = exports.getOrCreateScopeItemType = exports.getOrCreateScope = exports.assertGetOrCreateNodeScope = exports.allForAnyUnion = void 0;
const PQP = require("@microsoft/powerquery-parser");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const externalType_1 = require("../../../externalType");
const __1 = require("../../..");
const scope_1 = require("../../scope");
const inspectTypeConstant_1 = require("./inspectTypeConstant");
const inspectTypeEachExpression_1 = require("./inspectTypeEachExpression");
const inspectTypeErrorHandlingExpression_1 = require("./inspectTypeErrorHandlingExpression");
const inspectTypeFieldProjection_1 = require("./inspectTypeField/inspectTypeFieldProjection");
const inspectTypeFieldSelector_1 = require("./inspectTypeField/inspectTypeFieldSelector");
const inspectTypeFieldSpecification_1 = require("./inspectTypeFieldSpecification");
const inspectTypeFunctionExpression_1 = require("./inspectTypeFunctionExpression");
const inspectTypeFunctionType_1 = require("./inspectTypeFunctionType");
const inspectTypeIdentifier_1 = require("./inspectTypeIdentifier");
const inspectTypeIdentifierExpression_1 = require("./inspectTypeIdentifierExpression");
const inspectTypeIfExpression_1 = require("./inspectTypeIfExpression");
const inspectTypeInvokeExpression_1 = require("./inspectTypeInvokeExpression");
const inspectTypeList_1 = require("./inspectTypeList");
const inspectTypeListType_1 = require("./inspectTypeListType");
const inspectTypeLiteralExpression_1 = require("./inspectTypeLiteralExpression");
const inspectTypeNullCoalescingExpression_1 = require("./inspectTypeNullCoalescingExpression");
const inspectTypeParameter_1 = require("./inspectTypeParameter");
const inspectTypePrimitiveType_1 = require("./inspectTypePrimitiveType");
const inspectTypeRangeExpression_1 = require("./inspectTypeRangeExpression");
const inspectTypeRecord_1 = require("./inspectTypeRecord");
const inspectTypeRecordType_1 = require("./inspectTypeRecordType");
const inspectTypeRecursivePrimaryExpression_1 = require("./inspectTypeRecursivePrimaryExpression");
const inspectTypeTableType_1 = require("./inspectTypeTableType");
const inspectTypeTBinOpExpression_1 = require("./inspectTypeTBinOpExpression");
const inspectTypeUnaryExpression_1 = require("./inspectTypeUnaryExpression");
const deferenceIdentifier_1 = require("../../deferenceIdentifier");
// Recursively flattens all AnyUnion.unionedTypePairs into a single array,
// maps each entry into a boolean,
// then calls all(...) on the mapped values.
function allForAnyUnion(anyUnion, predicate) {
    return (anyUnion.unionedTypePairs
        .map((type) => type.extendedKind === language_1.Type.ExtendedTypeKind.AnyUnion
        ? allForAnyUnion(type, predicate)
        : predicate(type))
        .indexOf(false) === -1);
}
exports.allForAnyUnion = allForAnyUnion;
function assertGetOrCreateNodeScope(state, nodeId, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectScope, assertGetOrCreateNodeScope.name, correlationId);
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const triedGetOrCreateScope = yield getOrCreateScope(state, nodeId, trace.id);
        if (powerquery_parser_1.ResultUtils.isError(triedGetOrCreateScope)) {
            trace.exit({ [trace_1.TraceConstant.IsThrowing]: true });
            throw triedGetOrCreateScope.error;
        }
        trace.exit();
        return powerquery_parser_1.Assert.asDefined(triedGetOrCreateScope.value);
    });
}
exports.assertGetOrCreateNodeScope = assertGetOrCreateNodeScope;
function getOrCreateScope(state, nodeId, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectScope, getOrCreateScope.name, correlationId);
        const updatedState = Object.assign(Object.assign({}, state), { initialCorrelationId: trace.id });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const nodeScope = updatedState.scopeById.get(nodeId);
        if (nodeScope !== undefined) {
            trace.exit();
            return powerquery_parser_1.ResultUtils.ok(nodeScope);
        }
        const result = yield (0, scope_1.tryNodeScope)(updatedState, state.nodeIdMapCollection, state.eachScopeById, nodeId, state.scopeById);
        trace.exit();
        return result;
    });
}
exports.getOrCreateScope = getOrCreateScope;
function getOrCreateScopeItemType(state, scopeItem) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectScopeItem, getOrCreateScopeItemType.name, state.initialCorrelationId);
        const nodeId = scopeItem.id;
        const type = state.typeById.get(nodeId);
        if (type !== undefined) {
            trace.exit({ cacheHit: true });
            return type;
        }
        const scopeType = yield inspectScopeItem(state, scopeItem, trace.id);
        trace.exit();
        return scopeType;
    });
}
exports.getOrCreateScopeItemType = getOrCreateScopeItemType;
function inspectScopeItem(state, scopeItem, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        switch (scopeItem.kind) {
            case scope_1.ScopeItemKind.LetVariable:
            case scope_1.ScopeItemKind.RecordField:
            case scope_1.ScopeItemKind.SectionMember:
                return scopeItem.value === undefined
                    ? language_1.Type.UnknownInstance
                    : yield inspectXor(state, scopeItem.value, correlationId);
            case scope_1.ScopeItemKind.Each:
                return yield inspectXor(state, scopeItem.eachExpression, correlationId);
            case scope_1.ScopeItemKind.Parameter:
                return createParameterType(scopeItem);
            case scope_1.ScopeItemKind.Undefined:
                return language_1.Type.UnknownInstance;
            default:
                throw powerquery_parser_1.Assert.isNever(scopeItem);
        }
    });
}
exports.inspectScopeItem = inspectScopeItem;
function inspectTypeFromChildAttributeIndex(state, parentXorNode, attributeIndex, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeFromChildAttributeIndex.name, correlationId, __1.TraceUtils.xorNodeDetails(parentXorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const xorNode = PQP.Parser.NodeIdMapUtils.nthChildXor(state.nodeIdMapCollection, parentXorNode.node.id, attributeIndex);
        const result = xorNode !== undefined ? yield inspectXor(state, xorNode, trace.id) : language_1.Type.UnknownInstance;
        trace.exit();
        return result;
    });
}
exports.inspectTypeFromChildAttributeIndex = inspectTypeFromChildAttributeIndex;
function inspectXor(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectXor.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const xorNodeId = xorNode.node.id;
        const cached = state.typeById.get(xorNodeId);
        if (cached !== undefined) {
            trace.exit();
            return Promise.resolve(cached);
        }
        let result;
        switch (xorNode.node.kind) {
            case language_1.Ast.NodeKind.ArrayWrapper:
            case language_1.Ast.NodeKind.FieldSpecificationList:
            case language_1.Ast.NodeKind.GeneralizedIdentifier:
            case language_1.Ast.NodeKind.GeneralizedIdentifierPairedAnyLiteral:
            case language_1.Ast.NodeKind.GeneralizedIdentifierPairedExpression:
            case language_1.Ast.NodeKind.IdentifierPairedExpression:
            case language_1.Ast.NodeKind.ParameterList:
            case language_1.Ast.NodeKind.Section:
                trace.exit();
                return Promise.resolve(language_1.Type.NotApplicableInstance);
            case language_1.Ast.NodeKind.AsType:
            case language_1.Ast.NodeKind.AsNullablePrimitiveType:
            case language_1.Ast.NodeKind.CatchExpression:
            case language_1.Ast.NodeKind.FieldTypeSpecification:
            case language_1.Ast.NodeKind.OtherwiseExpression:
            case language_1.Ast.NodeKind.ParenthesizedExpression:
            case language_1.Ast.NodeKind.TypePrimaryType:
                result = yield inspectTypeFromChildAttributeIndex(state, xorNode, 1, trace.id);
                break;
            case language_1.Ast.NodeKind.ArithmeticExpression:
            case language_1.Ast.NodeKind.EqualityExpression:
            case language_1.Ast.NodeKind.LogicalExpression:
            case language_1.Ast.NodeKind.RelationalExpression:
                result = yield (0, inspectTypeTBinOpExpression_1.inspectTypeTBinOpExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.AsExpression:
            case language_1.Ast.NodeKind.SectionMember:
                result = yield inspectTypeFromChildAttributeIndex(state, xorNode, 2, trace.id);
                break;
            case language_1.Ast.NodeKind.Csv:
            case language_1.Ast.NodeKind.MetadataExpression:
                result = yield inspectTypeFromChildAttributeIndex(state, xorNode, 0, trace.id);
                break;
            case language_1.Ast.NodeKind.ListExpression:
            case language_1.Ast.NodeKind.ListLiteral:
                result = yield (0, inspectTypeList_1.inspectTypeList)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.NullableType:
            case language_1.Ast.NodeKind.NullablePrimitiveType:
                result = Object.assign(Object.assign({}, (yield inspectTypeFromChildAttributeIndex(state, xorNode, 1, trace.id))), { isNullable: true });
                break;
            case language_1.Ast.NodeKind.RecordLiteral:
            case language_1.Ast.NodeKind.RecordExpression:
                result = yield (0, inspectTypeRecord_1.inspectTypeRecord)(state, xorNode, trace.id);
                break;
            // TODO: how should error raising be typed?
            case language_1.Ast.NodeKind.ErrorRaisingExpression:
                result = language_1.Type.AnyInstance;
                break;
            case language_1.Ast.NodeKind.Constant:
                result = (0, inspectTypeConstant_1.inspectTypeConstant)(xorNode);
                break;
            case language_1.Ast.NodeKind.EachExpression:
                result = yield (0, inspectTypeEachExpression_1.inspectTypeEachExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.ErrorHandlingExpression:
                result = yield (0, inspectTypeErrorHandlingExpression_1.inspectTypeErrorHandlingExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.FieldProjection:
                result = yield (0, inspectTypeFieldProjection_1.inspectTypeFieldProjection)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.FieldSelector:
                result = yield (0, inspectTypeFieldSelector_1.inspectTypeFieldSelector)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.FieldSpecification:
                result = yield (0, inspectTypeFieldSpecification_1.inspectTypeFieldSpecification)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.FunctionExpression:
                result = yield (0, inspectTypeFunctionExpression_1.inspectTypeFunctionExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.FunctionType:
                result = yield (0, inspectTypeFunctionType_1.inspectTypeFunctionType)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.Identifier:
                result = yield (0, inspectTypeIdentifier_1.inspectTypeIdentifier)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.IdentifierExpression:
                result = yield (0, inspectTypeIdentifierExpression_1.inspectTypeIdentifierExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.IfExpression:
                result = yield (0, inspectTypeIfExpression_1.inspectTypeIfExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.IsExpression:
                result = language_1.Type.LogicalInstance;
                break;
            case language_1.Ast.NodeKind.InvokeExpression:
                result = yield (0, inspectTypeInvokeExpression_1.inspectTypeInvokeExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.IsNullablePrimitiveType:
                result = language_1.Type.LogicalInstance;
                break;
            case language_1.Ast.NodeKind.ItemAccessExpression:
                result = language_1.Type.AnyInstance;
                break;
            case language_1.Ast.NodeKind.LetExpression:
                result = yield inspectTypeFromChildAttributeIndex(state, xorNode, 3, trace.id);
                break;
            case language_1.Ast.NodeKind.ListType:
                result = yield (0, inspectTypeListType_1.inspectTypeListType)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.LiteralExpression:
                result = (0, inspectTypeLiteralExpression_1.inspectTypeLiteralExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.NotImplementedExpression:
                result = language_1.Type.NoneInstance;
                break;
            case language_1.Ast.NodeKind.NullCoalescingExpression:
                result = yield (0, inspectTypeNullCoalescingExpression_1.inspectTypeNullCoalescingExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.Parameter:
                result = yield (0, inspectTypeParameter_1.inspectTypeParameter)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.PrimitiveType:
                result = (0, inspectTypePrimitiveType_1.inspectTypePrimitiveType)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.RangeExpression:
                result = yield (0, inspectTypeRangeExpression_1.inspectTypeRangeExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.RecordType:
                result = yield (0, inspectTypeRecordType_1.inspectTypeRecordType)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.RecursivePrimaryExpression:
                result = yield (0, inspectTypeRecursivePrimaryExpression_1.inspectTypeRecursivePrimaryExpression)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.TableType:
                result = yield (0, inspectTypeTableType_1.inspectTypeTableType)(state, xorNode, trace.id);
                break;
            case language_1.Ast.NodeKind.UnaryExpression:
                result = yield (0, inspectTypeUnaryExpression_1.inspectTypeUnaryExpression)(state, xorNode, trace.id);
                break;
            default:
                throw powerquery_parser_1.Assert.isNever(xorNode.node);
        }
        state.typeById.set(xorNodeId, result);
        trace.exit({ resultKind: result.kind, extendedKind: result.extendedKind });
        return result;
    });
}
exports.inspectXor = inspectXor;
function dereferencedIdentifierType(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, dereferencedIdentifierType.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const updatedSettings = Object.assign(Object.assign({}, state), { initialCorrelationId: trace.id });
        const triedDeference = yield (0, deferenceIdentifier_1.tryDeferenceIdentifier)(updatedSettings, state.nodeIdMapCollection, state.eachScopeById, xorNode, state.scopeById);
        if (powerquery_parser_1.ResultUtils.isError(triedDeference)) {
            trace.exit({ [trace_1.TraceConstant.IsThrowing]: true });
            throw triedDeference.error;
        }
        else if (triedDeference.value === undefined) {
            trace.exit();
            return undefined;
        }
        const dereferencedLiteral = parser_1.XorNodeUtils.identifierExpressionLiteral(triedDeference.value);
        if (dereferencedLiteral === undefined) {
            trace.exit();
            return undefined;
        }
        const deferencedLiteral = dereferencedLiteral;
        const nodeScope = yield assertGetOrCreateNodeScope(state, triedDeference.value.node.id, trace.id);
        // When referencing an identifier as a recursive identifier there's no requirements
        // for it to resolve to a recursive reference.
        // This if it's a recursive identifier we need to also try the identifier without the recursive `@` prefix.
        let scopeItem = nodeScope.get(deferencedLiteral);
        if (deferencedLiteral.startsWith("@") && scopeItem === undefined) {
            scopeItem = nodeScope.get(deferencedLiteral.slice(1));
        }
        if (deferencedLiteral === "_" && (scopeItem === null || scopeItem === void 0 ? void 0 : scopeItem.kind) === scope_1.ScopeItemKind.Each) {
            return language_1.Type.UnknownInstance;
        }
        // The deferenced identifier can't be resolved within the local scope.
        // It either is either an invalid identifier or an external identifier (e.g `Odbc.Database`).
        if (scopeItem === undefined) {
            const request = externalType_1.ExternalTypeUtils.valueTypeRequest(deferencedLiteral);
            const result = state.library.externalTypeResolver(request);
            trace.exit();
            return result;
        }
        let nextXorNode;
        switch (scopeItem.kind) {
            case scope_1.ScopeItemKind.LetVariable:
            case scope_1.ScopeItemKind.RecordField:
            case scope_1.ScopeItemKind.SectionMember:
                nextXorNode = scopeItem.value;
                break;
            case scope_1.ScopeItemKind.Each:
                nextXorNode = scopeItem.eachExpression;
                break;
            case scope_1.ScopeItemKind.Parameter:
                return createParameterType(scopeItem);
            case scope_1.ScopeItemKind.Undefined:
                trace.exit();
                return undefined;
            default:
                throw powerquery_parser_1.Assert.isNever(scopeItem);
        }
        // Infinite recursion on an inclusive identifier.
        // There's no good way to handle the type of this as it requires evaluation, so mark it as any.
        if (deferencedLiteral.startsWith("@") && (nextXorNode === null || nextXorNode === void 0 ? void 0 : nextXorNode.node.id) === xorNode.node.id) {
            return language_1.Type.AnyInstance;
        }
        const result = nextXorNode
            ? yield inspectXor(state, nextXorNode, trace.id)
            : undefined;
        trace.exit();
        return result;
    });
}
exports.dereferencedIdentifierType = dereferencedIdentifierType;
function createParameterType(parameter) {
    return {
        kind: parameter.type ? language_1.TypeUtils.typeKindFromPrimitiveTypeConstantKind(parameter.type) : language_1.Type.TypeKind.Any,
        extendedKind: undefined,
        isNullable: parameter.isNullable || parameter.isOptional,
    };
}
exports.createParameterType = createParameterType;
//# sourceMappingURL=common.js.map