"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.examineFieldSpecificationList = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const __1 = require("../../..");
const inspectTypeFieldSpecification_1 = require("./inspectTypeFieldSpecification");
// It's called an examination instead of inspection because it doesn't return TType.
function examineFieldSpecificationList(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, examineFieldSpecificationList.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.FieldSpecificationList);
        const nodeIdMapCollection = state.nodeIdMapCollection;
        const fields = yield powerquery_parser_1.ArrayUtils.mapAsync(parser_1.NodeIdMapIterator.iterFieldSpecificationList(nodeIdMapCollection, xorNode), (fieldSpecification) => __awaiter(this, void 0, void 0, function* () {
            return [
                fieldSpecification.normalizedKeyLiteral,
                yield (0, inspectTypeFieldSpecification_1.inspectTypeFieldSpecification)(state, fieldSpecification.source, trace.id),
            ];
        }));
        const isOpen = parser_1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, xorNode.node.id, 3, language_1.Ast.NodeKind.Constant) !== undefined;
        const result = {
            fields: new Map(fields),
            isOpen,
        };
        trace.exit();
        return result;
    });
}
exports.examineFieldSpecificationList = examineFieldSpecificationList;
//# sourceMappingURL=examineFieldSpecificationList.js.map