"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeConstant = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
function inspectTypeConstant(xorNode) {
    parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.Constant);
    if (parser_1.XorNodeUtils.isContext(xorNode)) {
        return language_1.Type.UnknownInstance;
    }
    const constant = xorNode.node;
    switch (constant.constantKind) {
        case language_1.Constant.PrimitiveTypeConstant.Action:
            return language_1.Type.ActionInstance;
        case language_1.Constant.PrimitiveTypeConstant.Any:
            return language_1.Type.AnyInstance;
        case language_1.Constant.PrimitiveTypeConstant.AnyNonNull:
            return language_1.Type.AnyNonNullInstance;
        case language_1.Constant.PrimitiveTypeConstant.Binary:
            return language_1.Type.BinaryInstance;
        case language_1.Constant.PrimitiveTypeConstant.Date:
            return language_1.Type.DateInstance;
        case language_1.Constant.PrimitiveTypeConstant.DateTime:
            return language_1.Type.DateTimeInstance;
        case language_1.Constant.PrimitiveTypeConstant.DateTimeZone:
            return language_1.Type.DateTimeZoneInstance;
        case language_1.Constant.PrimitiveTypeConstant.Duration:
            return language_1.Type.DurationInstance;
        case language_1.Constant.PrimitiveTypeConstant.Function:
            return language_1.Type.FunctionInstance;
        case language_1.Constant.PrimitiveTypeConstant.List:
            return language_1.Type.ListInstance;
        case language_1.Constant.PrimitiveTypeConstant.Logical:
            return language_1.Type.LogicalInstance;
        case language_1.Constant.PrimitiveTypeConstant.None:
            return language_1.Type.NoneInstance;
        case language_1.Constant.PrimitiveTypeConstant.Null:
            return language_1.Type.NoneInstance;
        case language_1.Constant.PrimitiveTypeConstant.Number:
            return language_1.Type.NumberInstance;
        case language_1.Constant.PrimitiveTypeConstant.Record:
            return language_1.Type.RecordInstance;
        case language_1.Constant.PrimitiveTypeConstant.Table:
            return language_1.Type.TableInstance;
        case language_1.Constant.PrimitiveTypeConstant.Text:
            return language_1.Type.TextInstance;
        case language_1.Constant.PrimitiveTypeConstant.Time:
            return language_1.Type.TimeInstance;
        case language_1.Constant.PrimitiveTypeConstant.Type:
            return language_1.Type.TypePrimitiveInstance;
        default:
            return language_1.Type.UnknownInstance;
    }
}
exports.inspectTypeConstant = inspectTypeConstant;
//# sourceMappingURL=inspectTypeConstant.js.map