"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeErrorHandlingExpression = void 0;
const PQP = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const __1 = require("../../..");
const common_1 = require("./common");
function inspectTypeErrorHandlingExpression(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeErrorHandlingExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.ErrorHandlingExpression);
        // Grabs Ast.ErrorHandlingExpression.handler
        const handler = parser_1.NodeIdMapUtils.nthChildXorChecked(state.nodeIdMapCollection, xorNode.node.id, 2, [language_1.Ast.NodeKind.CatchExpression, language_1.Ast.NodeKind.OtherwiseExpression]);
        let errorHandlerResult;
        // No handler exists, i.e. `try 0`
        if (handler === undefined) {
            errorHandlerResult = language_1.Type.RecordInstance;
        }
        // If it's a catch handler, i.e. `try 1/0 catch () => "div by 0"`
        else if (handler.node.kind === language_1.Ast.NodeKind.CatchExpression) {
            // We care about the evaluation of the function,
            // and as typing isn't allowed on a catch expression it requires an inspection on the function's body
            const fnExpression = parser_1.NodeIdMapUtils.nthChildXorChecked(state.nodeIdMapCollection, handler.node.id, 1, [
                language_1.Ast.NodeKind.FunctionExpression,
            ]);
            // If there's no function expression, i.e. `try 1/0 catch`
            if (fnExpression === undefined) {
                errorHandlerResult = language_1.Type.UnknownInstance;
            }
            // Else examine Ast.FunctionExpression.expression
            else {
                errorHandlerResult = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, fnExpression, 3, trace.id);
            }
        }
        // Much easier here, simply return the paired expression.
        else if (handler.node.kind === language_1.Ast.NodeKind.OtherwiseExpression) {
            errorHandlerResult = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, handler, 1, trace.id);
        }
        else {
            throw new PQP.CommonError.InvariantError(`should never be reached`);
        }
        const result = language_1.TypeUtils.anyUnion([yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 1, trace.id), errorHandlerResult], state.traceManager, trace.id);
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeErrorHandlingExpression = inspectTypeErrorHandlingExpression;
//# sourceMappingURL=inspectTypeErrorHandlingExpression.js.map