"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectFieldType = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const common_1 = require("../common");
const scope_1 = require("../../../scope");
const __1 = require("../../../..");
// A field selection/projection is an operation a target value,
// where the target is either an EachExpression or a RecursivePrimaryExpression.
// In the code below that target for the FieldSelector/FieldProjection is called the scope.
//
// In the case of EachExpression:
//  use whatever scope was provided in InspectionTypeState.eachScopeById, else Unknown
//
// In the case of RecursivePrimaryExpression:
//  the scope is the previous sibling's type, so use NodeUtils.assertRecursiveExpressionPreviousSibling
function inspectFieldType(state, xorNode, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectFieldType.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, [
            language_1.Ast.NodeKind.FieldProjection,
            language_1.Ast.NodeKind.FieldSelector,
        ]);
        // travels up the AST to see if we're in an EachExpression or RecursivePrimaryExpression.
        const eachScopeItem = yield findEachScopeItem(state, xorNode, trace.correlationId);
        let fieldType;
        // if the scope is an EachExpression
        if (eachScopeItem) {
            fieldType = eachScopeItem.implicitParameterType;
        }
        // else it must be a RecursivePrimaryExpression,
        // so grab the previous sibling of the FieldProjection/FieldSelector
        else {
            const previousSibling = parser_1.NodeIdMapUtils.assertRecursiveExpressionPreviousSibling(state.nodeIdMapCollection, xorNode.node.id);
            fieldType = yield (0, common_1.inspectXor)(state, previousSibling, trace.id);
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(fieldType) });
        return fieldType;
    });
}
exports.inspectFieldType = inspectFieldType;
function findEachScopeItem(state, xorNode, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const nodeIdMapCollection = state.nodeIdMapCollection;
        let parent = parser_1.NodeIdMapUtils.parentXor(nodeIdMapCollection, xorNode.node.id);
        while (parent) {
            if (parent.node.kind === language_1.Ast.NodeKind.EachExpression) {
                return scope_1.ScopeUtils.assertGetScopeItemChecked(
                // eslint-disable-next-line no-await-in-loop
                yield (0, common_1.assertGetOrCreateNodeScope)(state, xorNode.node.id, correlationId), "_", scope_1.ScopeItemKind.Each);
            }
            parent = parser_1.NodeIdMapUtils.parentXor(nodeIdMapCollection, parent.node.id);
        }
        return undefined;
    });
}
//# sourceMappingURL=common.js.map