"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeFieldProjection = void 0;
const PQP = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const __1 = require("../../../..");
const common_1 = require("./common");
function inspectTypeFieldProjection(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeFieldProjection.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.FieldProjection);
        const projectedFieldLiterals = parser_1.NodeIdMapIterator.iterFieldProjectionFieldLiterals(state.nodeIdMapCollection, xorNode);
        const fieldType = yield (0, common_1.inspectFieldType)(state, xorNode, trace.id);
        const isOptional = parser_1.NodeIdMapUtils.nthChildAstChecked(state.nodeIdMapCollection, xorNode.node.id, 3, language_1.Ast.NodeKind.Constant) !== undefined;
        let result;
        switch (fieldType.kind) {
            case language_1.Type.TypeKind.Any: {
                const projectedFields = new Map(projectedFieldLiterals.map((fieldName) => [fieldName, language_1.Type.AnyInstance]));
                result = {
                    kind: language_1.Type.TypeKind.Any,
                    extendedKind: language_1.Type.ExtendedTypeKind.AnyUnion,
                    isNullable: fieldType.isNullable,
                    unionedTypePairs: [
                        {
                            kind: language_1.Type.TypeKind.Record,
                            extendedKind: language_1.Type.ExtendedTypeKind.DefinedRecord,
                            isNullable: fieldType.isNullable,
                            fields: projectedFields,
                            isOpen: false,
                        },
                        {
                            kind: language_1.Type.TypeKind.Table,
                            extendedKind: language_1.Type.ExtendedTypeKind.DefinedTable,
                            isNullable: fieldType.isNullable,
                            fields: new PQP.OrderedMap([...projectedFields]),
                            isOpen: false,
                        },
                    ],
                };
                break;
            }
            case language_1.Type.TypeKind.Record:
            case language_1.Type.TypeKind.Table:
                result = inspectRecordOrTableProjection(fieldType, projectedFieldLiterals, isOptional);
                break;
            case language_1.Type.TypeKind.Unknown:
                result = language_1.Type.UnknownInstance;
                break;
            default:
                result = language_1.Type.NoneInstance;
                break;
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeFieldProjection = inspectTypeFieldProjection;
function inspectRecordOrTableProjection(fieldType, projectedFieldLiterals, isOptional) {
    // All we know is fieldType was a Record/Table.
    // Create a DefinedRecord/DefinedTable with the projected fields.
    if (language_1.TypeUtils.isDefinedRecord(fieldType)) {
        return reducedFieldsToKeys(fieldType, projectedFieldLiterals, isOptional, reducedRecordFields);
    }
    else if (language_1.TypeUtils.isDefinedTable(fieldType)) {
        return reducedFieldsToKeys(fieldType, projectedFieldLiterals, isOptional, reducedTableFields);
    }
    else {
        const newFields = new Map(projectedFieldLiterals.map((fieldName) => [fieldName, language_1.Type.AnyInstance]));
        return fieldType.kind === language_1.Type.TypeKind.Record
            ? language_1.TypeUtils.definedRecord(false, newFields, false)
            : language_1.TypeUtils.definedTable(false, new PQP.OrderedMap([...newFields]), false);
    }
}
// Returns a subset of `current` using `keys`.
// If a mismatch is found it either returns Null if isOptional, else None.
function reducedFieldsToKeys(current, keys, isOptional, fieldsFactory) {
    const currentFieldNames = [...current.fields.keys()];
    if (!current.isOpen && !PQP.ArrayUtils.isSubset(currentFieldNames, keys)) {
        return isOptional ? language_1.Type.NullInstance : language_1.Type.NoneInstance;
    }
    return Object.assign(Object.assign({}, current), { fields: fieldsFactory(current, keys), isOpen: false });
}
function reducedRecordFields(current, keys) {
    return PQP.MapUtils.pick(current.fields, keys);
}
function reducedTableFields(current, keys) {
    return new PQP.OrderedMap([...PQP.MapUtils.pick(current.fields, keys).entries()]);
}
//# sourceMappingURL=inspectTypeFieldProjection.js.map