"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeFieldSelector = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const __1 = require("../../../..");
const common_1 = require("./common");
// A field selection/projection is an operation done on some value.
// The target can be either an EachExpression or a RecursivePrimaryExpression.
// In the code below that target for the FieldSelector/FieldProjection is called the scope.
//
// In the case of EachExpression:
//  use whatever scope was provided in InspectionTypeState.eachScopeById, else Unknown
//
// In the case of RecursivePrimaryExpression:
//  the scope is the previous sibling's type, so use NodeUtils.assertRecursiveExpressionPreviousSibling
function inspectTypeFieldSelector(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeFieldSelector.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.FieldSelector);
        const fieldName = parser_1.NodeIdMapUtils.wrappedContentAstChecked(state.nodeIdMapCollection, xorNode.node.id, language_1.Ast.NodeKind.GeneralizedIdentifier);
        if (fieldName === undefined) {
            trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(language_1.Type.UnknownInstance) });
            return language_1.Type.UnknownInstance;
        }
        const fieldType = yield (0, common_1.inspectFieldType)(state, xorNode, trace.id);
        const isOptional = parser_1.NodeIdMapUtils.nthChildAstChecked(state.nodeIdMapCollection, xorNode.node.id, 3, language_1.Ast.NodeKind.Constant) !== undefined;
        const result = getFieldSelectorType(fieldType, fieldName.literal, isOptional);
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeFieldSelector = inspectTypeFieldSelector;
function getFieldSelectorType(fieldType, fieldName, isOptional) {
    switch (fieldType.kind) {
        case language_1.Type.TypeKind.Any:
            return language_1.Type.AnyInstance;
        case language_1.Type.TypeKind.Unknown:
            return language_1.Type.UnknownInstance;
        case language_1.Type.TypeKind.Record:
        case language_1.Type.TypeKind.Table:
            return inspectRecordOrTable(fieldType, fieldName, isOptional);
        default:
            return language_1.Type.NoneInstance;
    }
}
function inspectRecordOrTable(fieldType, fieldName, isOptional) {
    if (fieldType.extendedKind === undefined) {
        return language_1.Type.AnyInstance;
    }
    const namedField = fieldType.fields.get(fieldName);
    if (namedField !== undefined) {
        return namedField;
    }
    else if (fieldType.isOpen) {
        return language_1.Type.AnyInstance;
    }
    else {
        return isOptional ? language_1.Type.NullInstance : language_1.Type.NoneInstance;
    }
}
//# sourceMappingURL=inspectTypeFieldSelector.js.map