"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeFunctionExpression = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const common_1 = require("./common");
const __1 = require("../../..");
const pseudoFunctionExpressionType_1 = require("../../pseudoFunctionExpressionType");
const inspectionSettings_1 = require("../../../inspectionSettings");
function inspectTypeFunctionExpression(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeFunctionExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.FunctionExpression);
        const pseudoType = (0, pseudoFunctionExpressionType_1.pseudoFunctionExpressionType)(state.nodeIdMapCollection, xorNode);
        const pseudoReturnType = pseudoType.returnType;
        let result;
        switch (state.typeStrategy) {
            case inspectionSettings_1.TypeStrategy.Extended: {
                const expressionType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 3, trace.id);
                // FunctionExpression.functionReturnType doesn't always match FunctionExpression.expression.
                // By examining the expression we might get a more accurate return type (eg. Function vs DefinedFunction),
                // or discover an error (eg. functionReturnType is Number but expression is Text).
                let returnType;
                // If the stated return type is Any,
                // then it might as well be the expression's type as it can't be any wider than Any.
                if (pseudoReturnType.kind === language_1.Type.TypeKind.Any) {
                    returnType = expressionType;
                }
                // If the return type is Any then see if we can narrow it to the stated return Type.
                else if (expressionType.kind === language_1.Type.TypeKind.Any &&
                    expressionType.extendedKind === language_1.Type.ExtendedTypeKind.AnyUnion &&
                    (0, common_1.allForAnyUnion)(expressionType, (type) => type.kind === pseudoReturnType.kind || type.kind === language_1.Type.TypeKind.Any)) {
                    returnType = expressionType;
                }
                // If the stated return type doesn't match the expression's type then it's None.
                else if (pseudoReturnType.kind !== expressionType.kind) {
                    trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(language_1.Type.NoneInstance) });
                    return language_1.Type.NoneInstance;
                }
                // If the expression's type can't be known, then assume it's the stated return Type.
                else if (expressionType.kind === language_1.Type.TypeKind.Unknown) {
                    returnType = pseudoReturnType;
                }
                // Else fallback to the expression's Type.
                else {
                    returnType = expressionType;
                }
                result = {
                    kind: language_1.Type.TypeKind.Function,
                    extendedKind: language_1.Type.ExtendedTypeKind.DefinedFunction,
                    isNullable: false,
                    parameters: pseudoType.parameters.map((pseudoParameter) => ({
                        nameLiteral: pseudoParameter.name.literal,
                        isNullable: pseudoParameter.isNullable,
                        isOptional: pseudoParameter.isOptional,
                        type: pseudoParameter.type,
                    })),
                    returnType,
                };
                break;
            }
            case inspectionSettings_1.TypeStrategy.Primitive:
                result = pseudoReturnType;
                break;
            default:
                powerquery_parser_1.Assert.isNever(state.typeStrategy);
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeFunctionExpression = inspectTypeFunctionExpression;
//# sourceMappingURL=inspectTypeFunctionExpression.js.map