"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeFunctionType = void 0;
const PQP = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const __1 = require("../../..");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const common_1 = require("./common");
const inspectionSettings_1 = require("../../../inspectionSettings");
function inspectTypeFunctionType(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeFunctionType.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.FunctionType);
        let result;
        switch (state.typeStrategy) {
            case inspectionSettings_1.TypeStrategy.Extended: {
                const parameters = parser_1.NodeIdMapUtils.nthChildXorChecked(state.nodeIdMapCollection, xorNode.node.id, 1, language_1.Ast.NodeKind.ParameterList);
                if (parameters === undefined) {
                    trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(language_1.Type.UnknownInstance) });
                    return language_1.Type.UnknownInstance;
                }
                const arrayWrapper = parser_1.NodeIdMapUtils.arrayWrapperContentXor(state.nodeIdMapCollection, parameters.node.id);
                if (arrayWrapper === undefined) {
                    trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(language_1.Type.UnknownInstance) });
                    return language_1.Type.UnknownInstance;
                }
                const parameterTypes = parser_1.NodeIdMapIterator.iterArrayWrapper(state.nodeIdMapCollection, arrayWrapper)
                    .map((parameter) => language_1.TypeUtils.inspectParameter(state.nodeIdMapCollection, parser_1.XorNodeUtils.assertAsNodeKind(parameter, language_1.Ast.NodeKind.Parameter)))
                    .filter(PQP.TypeScriptUtils.isDefined);
                const returnType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 2, trace.id);
                result = {
                    kind: language_1.Type.TypeKind.Type,
                    extendedKind: language_1.Type.ExtendedTypeKind.FunctionType,
                    isNullable: false,
                    parameters: parameterTypes,
                    returnType,
                };
                break;
            }
            case inspectionSettings_1.TypeStrategy.Primitive:
                result = language_1.Type.TypePrimitiveInstance;
                break;
            default:
                powerquery_parser_1.Assert.isNever(state.typeStrategy);
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeFunctionType = inspectTypeFunctionType;
//# sourceMappingURL=inspectTypeFunctionType.js.map