"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeIfExpression = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const common_1 = require("./common");
const __1 = require("../../..");
function inspectTypeIfExpression(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeIfExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.IfExpression);
        const conditionType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 1, trace.id);
        let result;
        if (conditionType.kind === language_1.Type.TypeKind.Unknown) {
            result = language_1.Type.UnknownInstance;
        }
        // Any is allowed so long as AnyUnion only contains Any or Logical.
        else if (conditionType.kind === language_1.Type.TypeKind.Any) {
            if (conditionType.extendedKind === language_1.Type.ExtendedTypeKind.AnyUnion &&
                !(0, common_1.allForAnyUnion)(conditionType, (type) => type.kind === language_1.Type.TypeKind.Logical || type.kind === language_1.Type.TypeKind.Any)) {
                result = language_1.Type.NoneInstance;
            }
            else {
                result = yield createAnyUnion(state, xorNode, trace.id);
            }
        }
        else if (conditionType.kind !== language_1.Type.TypeKind.Logical) {
            result = language_1.Type.NoneInstance;
        }
        else {
            result = yield createAnyUnion(state, xorNode, trace.id);
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeIfExpression = inspectTypeIfExpression;
function createAnyUnion(state, xorNode, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, createAnyUnion.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        const trueExprType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 3, trace.id);
        const falseExprType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 5, trace.id);
        const result = language_1.TypeUtils.anyUnion([trueExprType, falseExprType], state.traceManager, trace.id);
        trace.exit();
        return result;
    });
}
//# sourceMappingURL=inspectTypeIfExpression.js.map