"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeInvokeExpression = void 0;
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const externalType_1 = require("../../../externalType");
const __1 = require("../../..");
const common_1 = require("./common");
const deferenceIdentifier_1 = require("../../deferenceIdentifier");
function inspectTypeInvokeExpression(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeInvokeExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.InvokeExpression);
        const request = yield externalInvokeRequest(state, xorNode, trace.id);
        if (request !== undefined) {
            const type = state.library.externalTypeResolver(request);
            if (type !== undefined) {
                trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(type) });
                return type;
            }
        }
        const previousSibling = parser_1.NodeIdMapUtils.assertRecursiveExpressionPreviousSibling(state.nodeIdMapCollection, xorNode.node.id);
        const previousSiblingType = yield (0, common_1.inspectXor)(state, previousSibling, trace.id);
        let result;
        if (previousSiblingType.kind === language_1.Type.TypeKind.Any) {
            result = language_1.Type.AnyInstance;
        }
        else if (previousSiblingType.kind !== language_1.Type.TypeKind.Function) {
            result = language_1.Type.NoneInstance;
        }
        else if (previousSiblingType.extendedKind === language_1.Type.ExtendedTypeKind.DefinedFunction) {
            result = previousSiblingType.returnType;
        }
        else {
            result = language_1.Type.AnyInstance;
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeInvokeExpression = inspectTypeInvokeExpression;
function externalInvokeRequest(state, xorNode, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, externalInvokeRequest.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        const identifier = parser_1.NodeIdMapUtils.invokeExpressionIdentifier(state.nodeIdMapCollection, xorNode.node.id);
        if (identifier === undefined) {
            trace.exit();
            return undefined;
        }
        const updatedSettings = Object.assign(Object.assign({}, state), { initialCorrelationId: trace.id });
        const triedDeferencedIdentifier = yield (0, deferenceIdentifier_1.tryDeferenceIdentifier)(updatedSettings, state.nodeIdMapCollection, state.eachScopeById, identifier, state.scopeById);
        if (powerquery_parser_1.ResultUtils.isError(triedDeferencedIdentifier) || triedDeferencedIdentifier.value === undefined) {
            return undefined;
        }
        const types = [];
        for (const argument of parser_1.NodeIdMapIterator.iterInvokeExpression(state.nodeIdMapCollection, parser_1.XorNodeUtils.assertAsNodeKind(xorNode, language_1.Ast.NodeKind.InvokeExpression))) {
            // eslint-disable-next-line no-await-in-loop
            types.push(yield (0, common_1.inspectXor)(state, argument, trace.id));
        }
        const result = externalType_1.ExternalTypeUtils.invocationTypeRequest(powerquery_parser_1.Assert.asDefined(parser_1.XorNodeUtils.identifierExpressionLiteral(triedDeferencedIdentifier.value)), types);
        trace.exit();
        return result;
    });
}
//# sourceMappingURL=inspectTypeInvokeExpression.js.map