"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeList = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const __1 = require("../../..");
const common_1 = require("./common");
const inspectionSettings_1 = require("../../../inspectionSettings");
function inspectTypeList(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeList.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        let result;
        switch (state.typeStrategy) {
            case inspectionSettings_1.TypeStrategy.Extended: {
                (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
                const items = parser_1.NodeIdMapIterator.iterListItems(state.nodeIdMapCollection, xorNode);
                const elements = yield Promise.all(items.map((item) => (0, common_1.inspectXor)(state, item, trace.id)));
                result = {
                    kind: language_1.Type.TypeKind.List,
                    isNullable: false,
                    extendedKind: language_1.Type.ExtendedTypeKind.DefinedList,
                    elements,
                };
                break;
            }
            case inspectionSettings_1.TypeStrategy.Primitive:
                result = language_1.Type.ListInstance;
                break;
            default:
                powerquery_parser_1.Assert.isNever(state.typeStrategy);
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeList = inspectTypeList;
//# sourceMappingURL=inspectTypeList.js.map