"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeLiteralExpression = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const __1 = require("../../..");
function inspectTypeLiteralExpression(state, xorNode, correlationId) {
    const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeLiteralExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
    parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.LiteralExpression);
    let literalExpression;
    let typeKind;
    let result;
    switch (xorNode.kind) {
        case parser_1.XorNodeKind.Ast: {
            // We already checked it's a Ast Literal Expression.
            literalExpression = xorNode.node;
            typeKind = language_1.TypeUtils.typeKindFromLiteralKind(literalExpression.literalKind);
            switch (typeKind) {
                case language_1.Type.TypeKind.Number:
                    result = language_1.TypeUtils.numberLiteral(false, literalExpression.literal);
                    break;
                case language_1.Type.TypeKind.Text:
                    result = language_1.TypeUtils.textLiteral(false, literalExpression.literal);
                    break;
                default:
                    result = language_1.TypeUtils.primitiveType(literalExpression.literalKind === language_1.Ast.LiteralKind.Null, typeKind);
                    break;
            }
            break;
        }
        case parser_1.XorNodeKind.Context:
            result = language_1.Type.UnknownInstance;
            break;
        default:
            throw powerquery_parser_1.Assert.isNever(xorNode);
    }
    trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
    return result;
}
exports.inspectTypeLiteralExpression = inspectTypeLiteralExpression;
//# sourceMappingURL=inspectTypeLiteralExpression.js.map