"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeNullCoalescingExpression = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const __1 = require("../../..");
const common_1 = require("./common");
function inspectTypeNullCoalescingExpression(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeNullCoalescingExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.NullCoalescingExpression);
        const leftType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 0, trace.id);
        const nullCoalescingOperator = parser_1.NodeIdMapUtils.nthChildAstChecked(state.nodeIdMapCollection, xorNode.node.id, 1, language_1.Ast.NodeKind.Constant);
        let result;
        // '??' isn't present, treat it as an Expression.
        if (nullCoalescingOperator === undefined) {
            result = leftType;
        }
        else {
            const rightType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 2, trace.id);
            if (leftType.kind === language_1.Type.TypeKind.None || rightType.kind === language_1.Type.TypeKind.None) {
                result = language_1.Type.NoneInstance;
            }
            else if (!leftType.isNullable) {
                result = leftType;
            }
            else {
                result = language_1.TypeUtils.anyUnion([leftType, rightType], state.traceManager, trace.id);
            }
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeNullCoalescingExpression = inspectTypeNullCoalescingExpression;
//# sourceMappingURL=inspectTypeNullCoalescingExpression.js.map