"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeRangeExpression = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const __1 = require("../../..");
const common_1 = require("./common");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const inspectionSettings_1 = require("../../../inspectionSettings");
function inspectTypeRangeExpression(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeRangeExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.RangeExpression);
        let result;
        switch (state.typeStrategy) {
            case inspectionSettings_1.TypeStrategy.Extended: {
                const leftType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 0, trace.id);
                const rightType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 2, trace.id);
                if (leftType === undefined || rightType === undefined) {
                    result = language_1.Type.UnknownInstance;
                }
                else if (leftType.kind === language_1.Type.TypeKind.Number && rightType.kind === language_1.Type.TypeKind.Number) {
                    // TODO: handle isNullable better
                    if (leftType.isNullable === true || rightType.isNullable === true) {
                        result = language_1.Type.NoneInstance;
                    }
                    else {
                        result = language_1.Type.ListInstance;
                    }
                }
                else if (leftType.kind === language_1.Type.TypeKind.None || rightType.kind === language_1.Type.TypeKind.None) {
                    result = language_1.Type.NoneInstance;
                }
                else if (leftType.kind === language_1.Type.TypeKind.Unknown || rightType.kind === language_1.Type.TypeKind.Unknown) {
                    result = language_1.Type.UnknownInstance;
                }
                else {
                    result = language_1.Type.NoneInstance;
                }
                break;
            }
            case inspectionSettings_1.TypeStrategy.Primitive:
                result = language_1.Type.ListInstance;
                break;
            default:
                powerquery_parser_1.Assert.isNever(state.typeStrategy);
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeRangeExpression = inspectTypeRangeExpression;
//# sourceMappingURL=inspectTypeRangeExpression.js.map