"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeRecordType = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const __1 = require("../../..");
const examineFieldSpecificationList_1 = require("./examineFieldSpecificationList");
const inspectionSettings_1 = require("../../../inspectionSettings");
function inspectTypeRecordType(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeRecordType.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.RecordType);
        let result;
        switch (state.typeStrategy) {
            case inspectionSettings_1.TypeStrategy.Extended: {
                const fields = parser_1.NodeIdMapUtils.nthChildXorChecked(state.nodeIdMapCollection, xorNode.node.id, 0, language_1.Ast.NodeKind.FieldSpecificationList);
                if (fields === undefined) {
                    result = language_1.Type.UnknownInstance;
                }
                else {
                    result = Object.assign({ kind: language_1.Type.TypeKind.Type, extendedKind: language_1.Type.ExtendedTypeKind.RecordType, isNullable: false }, (yield (0, examineFieldSpecificationList_1.examineFieldSpecificationList)(state, fields, trace.id)));
                }
                break;
            }
            case inspectionSettings_1.TypeStrategy.Primitive:
                result = language_1.Type.TypePrimitiveInstance;
                break;
            default:
                powerquery_parser_1.Assert.isNever(state.typeStrategy);
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeRecordType = inspectTypeRecordType;
//# sourceMappingURL=inspectTypeRecordType.js.map