"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeRecursivePrimaryExpression = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const __1 = require("../../..");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const common_1 = require("./common");
function inspectTypeRecursivePrimaryExpression(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeRecursivePrimaryExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.RecursivePrimaryExpression);
        const head = parser_1.NodeIdMapUtils.nthChildXor(state.nodeIdMapCollection, xorNode.node.id, 0);
        if (head === undefined) {
            trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(language_1.Type.UnknownInstance) });
            return language_1.Type.UnknownInstance;
        }
        const headType = yield (0, common_1.inspectTypeFromChildAttributeIndex)(state, xorNode, 0, trace.id);
        if (headType.kind === language_1.Type.TypeKind.None || headType.kind === language_1.Type.TypeKind.Unknown) {
            trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(headType) });
            return headType;
        }
        const arrayWrapper = parser_1.NodeIdMapUtils.nthChildXorChecked(state.nodeIdMapCollection, xorNode.node.id, 1, language_1.Ast.NodeKind.ArrayWrapper);
        if (arrayWrapper === undefined) {
            trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(language_1.Type.UnknownInstance) });
            return language_1.Type.UnknownInstance;
        }
        const expressions = parser_1.NodeIdMapIterator.assertIterChildrenXor(state.nodeIdMapCollection, arrayWrapper.node.id);
        if (expressions === undefined) {
            trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(language_1.Type.UnknownInstance) });
            return language_1.Type.UnknownInstance;
        }
        let leftType = headType;
        for (const right of expressions) {
            // eslint-disable-next-line no-await-in-loop
            const rightType = yield (0, common_1.inspectXor)(state, right, trace.id);
            leftType = rightType;
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(leftType) });
        return leftType;
    });
}
exports.inspectTypeRecursivePrimaryExpression = inspectTypeRecursivePrimaryExpression;
//# sourceMappingURL=inspectTypeRecursivePrimaryExpression.js.map