"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeTableType = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const __1 = require("../../..");
const common_1 = require("./common");
const examineFieldSpecificationList_1 = require("./examineFieldSpecificationList");
const inspectionSettings_1 = require("../../../inspectionSettings");
function inspectTypeTableType(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeTableType.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.TableType);
        let result;
        switch (state.typeStrategy) {
            case inspectionSettings_1.TypeStrategy.Extended: {
                const rowType = parser_1.NodeIdMapUtils.nthChildXor(state.nodeIdMapCollection, xorNode.node.id, 1);
                if (rowType === undefined) {
                    result = language_1.Type.UnknownInstance;
                }
                else if (rowType.node.kind === language_1.Ast.NodeKind.FieldSpecificationList) {
                    result = Object.assign({ kind: language_1.Type.TypeKind.Type, extendedKind: language_1.Type.ExtendedTypeKind.TableType, isNullable: false }, (yield (0, examineFieldSpecificationList_1.examineFieldSpecificationList)(state, rowType, trace.id)));
                }
                else {
                    result = {
                        kind: language_1.Type.TypeKind.Type,
                        extendedKind: language_1.Type.ExtendedTypeKind.TableTypePrimaryExpression,
                        isNullable: false,
                        primaryExpression: yield (0, common_1.inspectXor)(state, rowType, trace.id),
                    };
                }
                break;
            }
            case inspectionSettings_1.TypeStrategy.Primitive:
                result = language_1.Type.TypePrimitiveInstance;
                break;
            default:
                powerquery_parser_1.Assert.isNever(state.typeStrategy);
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeTableType = inspectTypeTableType;
//# sourceMappingURL=inspectTypeTableType.js.map