"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.inspectTypeUnaryExpression = void 0;
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const __1 = require("../../..");
const common_1 = require("./common");
function inspectTypeUnaryExpression(state, xorNode, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectType, inspectTypeUnaryExpression.name, correlationId, __1.TraceUtils.xorNodeDetails(xorNode));
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parser_1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.UnaryExpression);
        const nodeIdMapCollection = state.nodeIdMapCollection;
        const unaryOperatorWrapper = parser_1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, xorNode.node.id, 0, language_1.Ast.NodeKind.ArrayWrapper);
        if (unaryOperatorWrapper === undefined) {
            trace.exit({ [trace_1.TraceConstant.Result]: language_1.Type.UnknownInstance });
            return language_1.Type.UnknownInstance;
        }
        let result;
        const expression = parser_1.NodeIdMapUtils.nthChildXor(nodeIdMapCollection, xorNode.node.id, 1);
        if (expression === undefined) {
            result = language_1.Type.UnknownInstance;
        }
        else {
            const expressionType = yield (0, common_1.inspectXor)(state, expression, trace.id);
            if (expressionType.kind === language_1.Type.TypeKind.Number) {
                result = inspectTypeUnaryNumber(state, expressionType, unaryOperatorWrapper.node.id);
            }
            else if (language_1.TypeUtils.isLogical(expressionType)) {
                result = inspectTypeUnaryLogical(state, expressionType, unaryOperatorWrapper.node.id);
            }
            else {
                result = language_1.Type.NoneInstance;
            }
        }
        trace.exit({ [trace_1.TraceConstant.Result]: __1.TraceUtils.typeDetails(result) });
        return result;
    });
}
exports.inspectTypeUnaryExpression = inspectTypeUnaryExpression;
function inspectTypeUnaryNumber(state, unaryExpressionType, unaryOperatorWrapperId) {
    const unaryNodeOperators = powerquery_parser_1.Assert.asDefined(parser_1.NodeIdMapIterator.iterChildrenAst(state.nodeIdMapCollection, unaryOperatorWrapperId));
    const expectedUnaryOperators = [
        language_1.Constant.UnaryOperator.Positive,
        language_1.Constant.UnaryOperator.Negative,
    ];
    const unaryOperators = [];
    let isPositive = true;
    for (const operator of unaryNodeOperators) {
        if (expectedUnaryOperators.indexOf(operator.constantKind) === -1) {
            return language_1.Type.NoneInstance;
        }
        unaryOperators.push(operator.constantKind);
        if (operator.constantKind === language_1.Constant.UnaryOperator.Negative) {
            isPositive = !isPositive;
        }
    }
    switch (unaryExpressionType.extendedKind) {
        case language_1.Type.ExtendedTypeKind.NumberLiteral:
            return language_1.TypeUtils.numberLiteral(unaryExpressionType.isNullable, [...unaryOperators, unaryExpressionType.literal].join(""));
        case undefined:
            return unaryExpressionType;
        default:
            throw powerquery_parser_1.Assert.isNever(unaryExpressionType);
    }
}
function inspectTypeUnaryLogical(state, unaryExpressionType, unaryOperatorWrapperId) {
    const unaryNodeOperators = powerquery_parser_1.Assert.asDefined(parser_1.NodeIdMapIterator.iterChildrenAst(state.nodeIdMapCollection, unaryOperatorWrapperId));
    for (const operator of unaryNodeOperators) {
        if (operator.constantKind !== language_1.Constant.UnaryOperator.Not) {
            return language_1.Type.NoneInstance;
        }
    }
    return unaryExpressionType;
}
//# sourceMappingURL=inspectTypeUnaryExpression.js.map