"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryType = exports.tryScopeType = void 0;
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const trace_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/common/trace");
const inspectType_1 = require("./inspectType");
const __1 = require("../..");
const typeCache_1 = require("../typeCache");
// eslint-disable-next-line require-await
function tryScopeType(settings, nodeIdMapCollection, nodeId, 
// If a TypeCache is given, then potentially add to its values and include it as part of the return,
// Else create a new TypeCache and include it in the return.
typeCache = typeCache_1.TypeCacheUtils.emptyCache()) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry(__1.InspectionTraceConstant.InspectScopeType, tryScopeType.name, settings.initialCorrelationId);
        const state = createState(settings, nodeIdMapCollection, typeCache, trace.id);
        const result = powerquery_parser_1.ResultUtils.ensureResultAsync(() => inspectScopeType(state, nodeId, trace.id), settings.locale);
        trace.exit();
        return result;
    });
}
exports.tryScopeType = tryScopeType;
function tryType(settings, nodeIdMapCollection, nodeId, 
// If a TypeCache is given, then potentially add to its values and include it as part of the return,
// Else create a new TypeCache and include it in the return.
typeCache = typeCache_1.TypeCacheUtils.emptyCache()) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry(__1.InspectionTraceConstant.InspectType, tryType.name, settings.initialCorrelationId);
        const state = createState(settings, nodeIdMapCollection, typeCache, trace.id);
        const result = yield powerquery_parser_1.ResultUtils.ensureResultAsync(() => (0, inspectType_1.inspectXor)(state, parser_1.NodeIdMapUtils.assertXor(nodeIdMapCollection, nodeId), trace.id), settings.locale);
        trace.exit({ [trace_1.TraceConstant.IsError]: result.kind });
        return result;
    });
}
exports.tryType = tryType;
function inspectScopeType(state, nodeId, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(__1.InspectionTraceConstant.InspectScopeType, inspectScopeType.name, correlationId);
        const nodeScope = yield (0, inspectType_1.assertGetOrCreateNodeScope)(state, nodeId, trace.id);
        for (const scopeItem of nodeScope.values()) {
            if (!state.typeById.has(scopeItem.id)) {
                // eslint-disable-next-line no-await-in-loop
                state.typeById.set(scopeItem.id, yield (0, inspectType_1.getOrCreateScopeItemType)(state, scopeItem));
            }
        }
        const result = new Map();
        for (const [key, scopeItem] of nodeScope.entries()) {
            const type = powerquery_parser_1.MapUtils.assertGet(state.typeById, scopeItem.id, `expected nodeId to be in givenTypeById`, { nodeId: scopeItem.id });
            result.set(key, type);
        }
        trace.exit();
        return result;
    });
}
function createState(settings, nodeIdMapCollection, typeCache, correlationId) {
    return {
        library: settings.library,
        locale: settings.locale,
        isWorkspaceCacheAllowed: settings.isWorkspaceCacheAllowed,
        cancellationToken: settings.cancellationToken,
        eachScopeById: settings.eachScopeById,
        initialCorrelationId: correlationId,
        traceManager: settings.traceManager,
        typeStrategy: settings.typeStrategy,
        typeById: typeCache.typeById,
        nodeIdMapCollection,
        scopeById: typeCache.scopeById,
    };
}
//# sourceMappingURL=task.js.map