"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSymbolForIdentifierPairedExpression = exports.getSymbolsForSection = exports.getSymbolsForRecord = exports.getSymbolsForLetExpression = exports.getSymbolKindFromNode = exports.getSymbolKindFromLiteralExpression = exports.getScopeItemKindText = exports.getIdentifierType = exports.getAutocompleteItemsFromScope = exports.inspectionSettings = void 0;
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const _1 = require(".");
const inspectionSettings_1 = require("./inspectionSettings");
const autocomplete_1 = require("./inspection/autocomplete");
const library_1 = require("./library");
function inspectionSettings(settings, overrides) {
    var _a, _b, _c, _d;
    return Object.assign(Object.assign({}, settings), { isWorkspaceCacheAllowed: (_a = overrides === null || overrides === void 0 ? void 0 : overrides.isWorkspaceCacheAllowed) !== null && _a !== void 0 ? _a : true, library: (_b = overrides === null || overrides === void 0 ? void 0 : overrides.library) !== null && _b !== void 0 ? _b : library_1.Library.NoOpLibrary, typeStrategy: (_c = overrides === null || overrides === void 0 ? void 0 : overrides.typeStrategy) !== null && _c !== void 0 ? _c : inspectionSettings_1.TypeStrategy.Extended, eachScopeById: (_d = overrides === null || overrides === void 0 ? void 0 : overrides.eachScopeById) !== null && _d !== void 0 ? _d : undefined });
}
exports.inspectionSettings = inspectionSettings;
function getAutocompleteItemsFromScope(context) {
    var _a;
    const triedNodeScope = context.triedNodeScope;
    const triedScopeType = context.triedScopeType;
    if (powerquery_parser_1.ResultUtils.isError(triedNodeScope)) {
        return [];
    }
    const nodeScope = triedNodeScope.value;
    const scopeTypeByKey = powerquery_parser_1.ResultUtils.okOrDefault(triedScopeType, new Map());
    const contextText = context.text;
    const partial = [];
    for (const [label, scopeItem] of nodeScope.entries()) {
        const autocompleteItem = autocomplete_1.AutocompleteItemUtils.fromScopeItem(label, scopeItem, (_a = scopeTypeByKey.get(label)) !== null && _a !== void 0 ? _a : language_1.Type.UnknownInstance, contextText);
        if (autocompleteItem) {
            partial.push(autocompleteItem);
        }
    }
    return partial;
}
exports.getAutocompleteItemsFromScope = getAutocompleteItemsFromScope;
function getIdentifierType(inspected, identifier) {
    return __awaiter(this, void 0, void 0, function* () {
        const triedScopeType = yield inspected.triedScopeType;
        return powerquery_parser_1.ResultUtils.isOk(triedScopeType) ? triedScopeType.value.get(identifier) : undefined;
    });
}
exports.getIdentifierType = getIdentifierType;
function getScopeItemKindText(scopeItemKind) {
    switch (scopeItemKind) {
        case _1.Inspection.ScopeItemKind.Each:
            return "each";
        case _1.Inspection.ScopeItemKind.LetVariable:
            return "let-variable";
        case _1.Inspection.ScopeItemKind.Parameter:
            return "parameter";
        case _1.Inspection.ScopeItemKind.RecordField:
            return "record-field";
        case _1.Inspection.ScopeItemKind.SectionMember:
            return "section-member";
        case _1.Inspection.ScopeItemKind.Undefined:
            return "unknown";
        default:
            throw powerquery_parser_1.Assert.isNever(scopeItemKind);
    }
}
exports.getScopeItemKindText = getScopeItemKindText;
function getSymbolKindFromLiteralExpression(node) {
    switch (node.literalKind) {
        case language_1.Ast.LiteralKind.List:
            return vscode_languageserver_types_1.SymbolKind.Array;
        case language_1.Ast.LiteralKind.Logical:
            return vscode_languageserver_types_1.SymbolKind.Boolean;
        case language_1.Ast.LiteralKind.Null:
            return vscode_languageserver_types_1.SymbolKind.Null;
        case language_1.Ast.LiteralKind.Numeric:
            return vscode_languageserver_types_1.SymbolKind.Number;
        case language_1.Ast.LiteralKind.Text:
            return vscode_languageserver_types_1.SymbolKind.String;
        default:
            return powerquery_parser_1.Assert.isNever(node.literalKind);
    }
}
exports.getSymbolKindFromLiteralExpression = getSymbolKindFromLiteralExpression;
function getSymbolKindFromNode(node) {
    switch (node.kind) {
        case language_1.Ast.NodeKind.Constant:
            return vscode_languageserver_types_1.SymbolKind.Constant;
        case language_1.Ast.NodeKind.FunctionExpression:
            return vscode_languageserver_types_1.SymbolKind.Function;
        case language_1.Ast.NodeKind.ListExpression:
        case language_1.Ast.NodeKind.ListLiteral:
            return vscode_languageserver_types_1.SymbolKind.Array;
        case language_1.Ast.NodeKind.LiteralExpression:
            return getSymbolKindFromLiteralExpression(node);
        case language_1.Ast.NodeKind.MetadataExpression:
            return vscode_languageserver_types_1.SymbolKind.TypeParameter;
        case language_1.Ast.NodeKind.RecordExpression:
        case language_1.Ast.NodeKind.RecordLiteral:
            return vscode_languageserver_types_1.SymbolKind.Struct;
        case language_1.Ast.NodeKind.Section:
            return vscode_languageserver_types_1.SymbolKind.Module;
        default:
            return vscode_languageserver_types_1.SymbolKind.Variable;
    }
}
exports.getSymbolKindFromNode = getSymbolKindFromNode;
function getSymbolsForLetExpression(expressionNode) {
    const documentSymbols = [];
    for (const element of expressionNode.variableList.elements) {
        const pairedExpression = element;
        const memberSymbol = getSymbolForIdentifierPairedExpression(pairedExpression.node);
        documentSymbols.push(memberSymbol);
    }
    return documentSymbols;
}
exports.getSymbolsForLetExpression = getSymbolsForLetExpression;
function getSymbolsForRecord(recordNode) {
    const documentSymbols = [];
    for (const element of recordNode.content.elements) {
        documentSymbols.push({
            kind: vscode_languageserver_types_1.SymbolKind.Field,
            deprecated: false,
            name: element.node.key.literal,
            range: _1.PositionUtils.rangeFromTokenRange(element.node.tokenRange),
            selectionRange: _1.PositionUtils.rangeFromTokenRange(element.node.key.tokenRange),
        });
    }
    return documentSymbols;
}
exports.getSymbolsForRecord = getSymbolsForRecord;
function getSymbolsForSection(sectionNode) {
    return sectionNode.sectionMembers.elements.map((sectionMember) => getSymbolForIdentifierPairedExpression(sectionMember.namePairedExpression));
}
exports.getSymbolsForSection = getSymbolsForSection;
function getSymbolForIdentifierPairedExpression(identifierPairedExpressionNode) {
    return {
        kind: getSymbolKindFromNode(identifierPairedExpressionNode.value),
        deprecated: false,
        name: identifierPairedExpressionNode.key.literal,
        range: _1.PositionUtils.rangeFromTokenRange(identifierPairedExpressionNode.tokenRange),
        selectionRange: _1.PositionUtils.rangeFromTokenRange(identifierPairedExpressionNode.key.tokenRange),
    };
}
exports.getSymbolForIdentifierPairedExpression = getSymbolForIdentifierPairedExpression;
//# sourceMappingURL=inspectionUtils.js.map