"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.parameterInformation = exports.signatureInformation = exports.isType = exports.isFunction = exports.isConstant = exports.hasDefinition = exports.getKeys = exports.getDefinition = exports.functionDefinition = exports.externalTypeResolver = exports.constantDefinition = exports.assertIsType = exports.assertIsFunction = exports.assertIsConstant = exports.assertAsType = exports.assertAsFunction = exports.assertAsConstant = void 0;
const library_1 = require("./library");
function assertAsConstant(definition) {
    assertIsConstant(definition);
    return definition;
}
exports.assertAsConstant = assertAsConstant;
function assertAsFunction(definition) {
    assertIsFunction(definition);
    return definition;
}
exports.assertAsFunction = assertAsFunction;
function assertAsType(definition) {
    assertIsType(definition);
    return definition;
}
exports.assertAsType = assertAsType;
function assertIsConstant(definition) {
    if (!isConstant(definition)) {
        throw new Error(`expected definition to be ${library_1.LibraryDefinitionKind.Constant}`);
    }
}
exports.assertIsConstant = assertIsConstant;
function assertIsFunction(definition) {
    if (!isFunction(definition)) {
        throw new Error(`expected definition to be ${library_1.LibraryDefinitionKind.Function}`);
    }
}
exports.assertIsFunction = assertIsFunction;
function assertIsType(definition) {
    if (!isType(definition)) {
        throw new Error(`expected definition to be ${library_1.LibraryDefinitionKind.Type}`);
    }
}
exports.assertIsType = assertIsType;
function constantDefinition(label, description, asType, completionItemKind) {
    return {
        kind: library_1.LibraryDefinitionKind.Constant,
        asPowerQueryType: asType,
        completionItemKind,
        description,
        label,
    };
}
exports.constantDefinition = constantDefinition;
function externalTypeResolver(libraryDefinitions) {
    return (request) => { var _a; return (_a = getDefinition(libraryDefinitions, request.identifierLiteral)) === null || _a === void 0 ? void 0 : _a.asPowerQueryType; };
}
exports.externalTypeResolver = externalTypeResolver;
function functionDefinition(label, description, asType, completionItemKind, parameters) {
    return {
        kind: library_1.LibraryDefinitionKind.Function,
        asPowerQueryType: asType,
        completionItemKind,
        description,
        label,
        parameters,
    };
}
exports.functionDefinition = functionDefinition;
function getDefinition(libraryDefinitions, identifierLiteral) {
    var _a;
    return ((_a = libraryDefinitions.staticLibraryDefinitions.get(identifierLiteral)) !== null && _a !== void 0 ? _a : libraryDefinitions.dynamicLibraryDefinitions().get(identifierLiteral));
}
exports.getDefinition = getDefinition;
function getKeys(libraryDefinitions) {
    return [
        ...libraryDefinitions.staticLibraryDefinitions.keys(),
        ...libraryDefinitions.dynamicLibraryDefinitions().keys(),
    ];
}
exports.getKeys = getKeys;
function hasDefinition(libraryDefinitions, key) {
    return getDefinition(libraryDefinitions, key) !== undefined;
}
exports.hasDefinition = hasDefinition;
function isConstant(definition) {
    return (definition === null || definition === void 0 ? void 0 : definition.kind) === library_1.LibraryDefinitionKind.Constant;
}
exports.isConstant = isConstant;
function isFunction(definition) {
    return (definition === null || definition === void 0 ? void 0 : definition.kind) === library_1.LibraryDefinitionKind.Function;
}
exports.isFunction = isFunction;
function isType(definition) {
    return (definition === null || definition === void 0 ? void 0 : definition.kind) === library_1.LibraryDefinitionKind.Type;
}
exports.isType = isType;
function signatureInformation(libraryFunctionSignature) {
    return {
        label: libraryFunctionSignature.label,
        documentation: libraryFunctionSignature.description,
        parameters: libraryFunctionSignature.parameters.map(parameterInformation),
    };
}
exports.signatureInformation = signatureInformation;
function parameterInformation(libraryParameter) {
    return {
        label: libraryParameter.label,
        documentation: undefined,
    };
}
exports.parameterInformation = parameterInformation;
//# sourceMappingURL=libraryDefinitionUtils.js.map