import { Result } from "@microsoft/powerquery-parser";
import { ExternalType } from "../externalType";
import { Library } from "../library";
import { LibrarySymbol } from "./librarySymbol";
export interface IncompleteLibrary {
    readonly library: Library.ILibrary;
    readonly failedLibrarySymbolConversions: ReadonlyArray<FailedLibrarySymbolConversion>;
}
export interface IncompleteLibraryDefinitions {
    readonly libraryDefinitions: ReadonlyMap<string, Library.TLibraryDefinition>;
    readonly failedLibrarySymbolConversions: ReadonlyArray<FailedLibrarySymbolConversion>;
}
export interface FailedLibrarySymbolConversion {
    readonly librarySymbol: LibrarySymbol;
    readonly kind: FailedLibrarySymbolConversionKind;
}
export interface FailedLibrarySymbolParameterConversion extends FailedLibrarySymbolConversion {
    readonly kind: FailedLibrarySymbolConversionKind.Parameter;
    readonly index: number;
}
export declare enum FailedLibrarySymbolConversionKind {
    CompletionItemKind = "CompletionItemKind",
    DefinedFunction = "DefinedFunction",
    Parameter = "Parameter",
    PrimitiveType = "PrimitiveType"
}
/**
    Always returns a library, even if there are failed library symbol conversions.
    An Ok result indicates that all library symbols were successfully converted.
    An Error result indicates that some library symbols failed to convert.
*/
export declare function createLibrary(librarySymbols: ReadonlyArray<LibrarySymbol>, dynamicLibraryDefinitions: () => ReadonlyMap<string, Library.TLibraryDefinition>, externalTypeResolverFn: ExternalType.TExternalTypeResolverFn | undefined): Result<Library.ILibrary, IncompleteLibrary>;
export declare function createLibraryDefinition(librarySymbol: LibrarySymbol): Result<Library.TLibraryDefinition, FailedLibrarySymbolConversion>;
export declare function createLibraryDefinitions(librarySymbols: ReadonlyArray<LibrarySymbol>): Result<ReadonlyMap<string, Library.TLibraryDefinition>, IncompleteLibraryDefinitions>;
