"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAfterTokenPosition = exports.isOnTokenPosition = exports.isBeforeTokenPosition = exports.isInToken = exports.isOnTokenEnd = exports.isOnTokenStart = exports.isAfterAst = exports.isOnAstEnd = exports.isOnAstStart = exports.isInAst = exports.isBeforeAst = exports.isAfterContext = exports.isOnContextEnd = exports.isOnContextStart = exports.isInContext = exports.isBeforeContext = exports.isAfterXor = exports.isOnXorEnd = exports.isOnXorStart = exports.isInXor = exports.isBeforeXor = exports.rangeFromTokenRange = exports.rangeFromTokenPositions = exports.rangeFromToken = exports.rangeFromXorNode = exports.positionFromTokenPosition = exports.compareTokenPosition = void 0;
const PQP = require("@microsoft/powerquery-parser");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const tokenPositionComparison_1 = require("./tokenPositionComparison");
function compareTokenPosition(position, tokenPosition) {
    if (isOnTokenPosition(position, tokenPosition)) {
        return tokenPositionComparison_1.TokenPositionComparison.OnToken;
    }
    else if (isAfterTokenPosition(position, tokenPosition, false)) {
        return tokenPositionComparison_1.TokenPositionComparison.RightOfToken;
    }
    else if (isBeforeTokenPosition(position, tokenPosition, false)) {
        return tokenPositionComparison_1.TokenPositionComparison.LeftOfToken;
    }
    else {
        throw new PQP.CommonError.InvariantError("position is not on, after, or before tokenPosition", {
            position,
            tokenPosition,
        });
    }
}
exports.compareTokenPosition = compareTokenPosition;
function positionFromTokenPosition(tokenPosition) {
    return {
        line: tokenPosition.lineNumber,
        character: tokenPosition.lineCodeUnit,
    };
}
exports.positionFromTokenPosition = positionFromTokenPosition;
// Attempts to turn a TXorNode into a Range.
// Returns undefined if there are no leafs nodes.
function rangeFromXorNode(nodeIdMapCollection, xorNode) {
    return __awaiter(this, void 0, void 0, function* () {
        const nodeId = xorNode.node.id;
        const leftMostLeaf = parser_1.NodeIdMapUtils.leftMostLeaf(nodeIdMapCollection, nodeId);
        const rightMostLeaf = yield parser_1.NodeIdMapUtils.rightMostLeaf(nodeIdMapCollection, nodeId);
        return Promise.resolve(leftMostLeaf === undefined || rightMostLeaf === undefined
            ? undefined
            : rangeFromTokenPositions(leftMostLeaf.tokenRange.positionStart, rightMostLeaf.tokenRange.positionEnd));
    });
}
exports.rangeFromXorNode = rangeFromXorNode;
function rangeFromToken(token) {
    return powerquery_parser_1.Assert.asDefined(rangeFromTokenPositions(token.positionStart, token.positionEnd));
}
exports.rangeFromToken = rangeFromToken;
function rangeFromTokenPositions(startTokenPosition, endTokenPosition) {
    if (startTokenPosition && endTokenPosition) {
        return {
            start: positionFromTokenPosition(startTokenPosition),
            end: positionFromTokenPosition(endTokenPosition),
        };
    }
    return undefined;
}
exports.rangeFromTokenPositions = rangeFromTokenPositions;
function rangeFromTokenRange(tokenRange) {
    return powerquery_parser_1.Assert.asDefined(rangeFromTokenPositions(tokenRange.positionStart, tokenRange.positionEnd));
}
exports.rangeFromTokenRange = rangeFromTokenRange;
function isBeforeXor(position, xorNode, isBoundIncluded) {
    switch (xorNode.kind) {
        case parser_1.XorNodeKind.Ast:
            return isBeforeAst(position, xorNode.node, isBoundIncluded);
        case parser_1.XorNodeKind.Context:
            return isBeforeContext(position, xorNode.node, isBoundIncluded);
        default:
            throw powerquery_parser_1.Assert.isNever(xorNode);
    }
}
exports.isBeforeXor = isBeforeXor;
function isInXor(nodeIdMapCollection, position, xorNode, isLowerBoundIncluded, isUpperBoundIncluded) {
    switch (xorNode.kind) {
        case parser_1.XorNodeKind.Ast:
            return isInAst(position, xorNode.node, isLowerBoundIncluded, isUpperBoundIncluded);
        case parser_1.XorNodeKind.Context:
            return isInContext(nodeIdMapCollection, position, xorNode.node, isLowerBoundIncluded, isUpperBoundIncluded);
        default:
            throw powerquery_parser_1.Assert.isNever(xorNode);
    }
}
exports.isInXor = isInXor;
function isOnXorStart(position, xorNode) {
    switch (xorNode.kind) {
        case parser_1.XorNodeKind.Ast:
            return isOnAstStart(position, xorNode.node);
        case parser_1.XorNodeKind.Context:
            return isOnContextStart(position, xorNode.node);
        default:
            throw powerquery_parser_1.Assert.isNever(xorNode);
    }
}
exports.isOnXorStart = isOnXorStart;
function isOnXorEnd(nodeIdMapCollection, position, xorNode) {
    switch (xorNode.kind) {
        case parser_1.XorNodeKind.Ast:
            return Promise.resolve(isOnAstEnd(position, xorNode.node));
        case parser_1.XorNodeKind.Context:
            return isOnContextEnd(nodeIdMapCollection, position, xorNode.node);
        default:
            throw powerquery_parser_1.Assert.isNever(xorNode);
    }
}
exports.isOnXorEnd = isOnXorEnd;
function isAfterXor(nodeIdMapCollection, position, xorNode, isBoundIncluded) {
    switch (xorNode.kind) {
        case parser_1.XorNodeKind.Ast:
            return Promise.resolve(isAfterAst(position, xorNode.node, isBoundIncluded));
        case parser_1.XorNodeKind.Context:
            return isAfterContext(nodeIdMapCollection, position, xorNode.node, isBoundIncluded);
        default:
            throw powerquery_parser_1.Assert.isNever(xorNode);
    }
}
exports.isAfterXor = isAfterXor;
function isBeforeContext(position, contextNode, isBoundIncluded) {
    const tokenStart = contextNode.tokenStart;
    if (tokenStart === undefined) {
        return false;
    }
    return isBeforeTokenPosition(position, tokenStart.positionStart, isBoundIncluded);
}
exports.isBeforeContext = isBeforeContext;
function isInContext(nodeIdMapCollection, position, contextNode, isLowerBoundIncluded, isHigherBoundIncluded) {
    return (!isBeforeContext(position, contextNode, isLowerBoundIncluded) &&
        !isAfterContext(nodeIdMapCollection, position, contextNode, isHigherBoundIncluded));
}
exports.isInContext = isInContext;
function isOnContextStart(position, contextNode) {
    return contextNode.tokenStart !== undefined
        ? isOnTokenPosition(position, contextNode.tokenStart.positionStart)
        : false;
}
exports.isOnContextStart = isOnContextStart;
function isOnContextEnd(nodeIdMapCollection, position, contextNode) {
    return __awaiter(this, void 0, void 0, function* () {
        const leaf = yield parser_1.NodeIdMapUtils.rightMostLeaf(nodeIdMapCollection, contextNode.id);
        if (leaf === undefined) {
            return Promise.resolve(false);
        }
        return Promise.resolve(isOnAstEnd(position, leaf));
    });
}
exports.isOnContextEnd = isOnContextEnd;
function isAfterContext(nodeIdMapCollection, position, contextNode, isBoundIncluded) {
    return __awaiter(this, void 0, void 0, function* () {
        const leaf = yield parser_1.NodeIdMapUtils.rightMostLeaf(nodeIdMapCollection, contextNode.id);
        if (leaf === undefined) {
            // We're assuming position is a valid range for the document.
            // Therefore if the context node didn't have a token (caused by EOF) we can make this assumption.
            if (contextNode.tokenStart === undefined) {
                return Promise.resolve(false);
            }
            else {
                return Promise.resolve(isAfterTokenPosition(position, contextNode.tokenStart.positionEnd, isBoundIncluded));
            }
        }
        return Promise.resolve(isAfterAst(position, leaf, isBoundIncluded));
    });
}
exports.isAfterContext = isAfterContext;
function isBeforeAst(position, astNode, isBoundIncluded) {
    return isBeforeTokenPosition(position, astNode.tokenRange.positionStart, isBoundIncluded);
}
exports.isBeforeAst = isBeforeAst;
function isInAst(position, astNode, isLowerBoundIncluded, isHigherBoundIncluded) {
    return (!isBeforeAst(position, astNode, isLowerBoundIncluded) && !isAfterAst(position, astNode, isHigherBoundIncluded));
}
exports.isInAst = isInAst;
function isOnAstStart(position, astNode) {
    return isOnTokenPosition(position, astNode.tokenRange.positionStart);
}
exports.isOnAstStart = isOnAstStart;
function isOnAstEnd(position, astNode) {
    return isOnTokenPosition(position, astNode.tokenRange.positionEnd);
}
exports.isOnAstEnd = isOnAstEnd;
function isAfterAst(position, astNode, isBoundIncluded) {
    return isAfterTokenPosition(position, astNode.tokenRange.positionEnd, isBoundIncluded);
}
exports.isAfterAst = isAfterAst;
function isOnTokenStart(position, token) {
    return position.line === token.positionStart.lineNumber && position.character === token.positionStart.lineCodeUnit;
}
exports.isOnTokenStart = isOnTokenStart;
function isOnTokenEnd(position, token) {
    return position.line === token.positionEnd.lineNumber && position.character === token.positionEnd.lineCodeUnit;
}
exports.isOnTokenEnd = isOnTokenEnd;
function isInToken(position, token, isLowerBoundIncluded, isHigherBoundIncluded) {
    return (!isBeforeTokenPosition(position, token.positionStart, isLowerBoundIncluded) &&
        !isAfterTokenPosition(position, token.positionEnd, isHigherBoundIncluded));
}
exports.isInToken = isInToken;
function isBeforeTokenPosition(position, tokenPosition, isBoundIncluded) {
    const positionLineNumber = position.line;
    if (positionLineNumber < tokenPosition.lineNumber) {
        return true;
    }
    else if (positionLineNumber > tokenPosition.lineNumber) {
        return false;
    }
    else {
        const upperBound = isBoundIncluded ? tokenPosition.lineCodeUnit : tokenPosition.lineCodeUnit + 1;
        return position.character < upperBound;
    }
}
exports.isBeforeTokenPosition = isBeforeTokenPosition;
function isOnTokenPosition(position, tokenPosition) {
    return position.line === tokenPosition.lineNumber && position.character === tokenPosition.lineCodeUnit;
}
exports.isOnTokenPosition = isOnTokenPosition;
function isAfterTokenPosition(position, tokenPosition, isBoundIncluded) {
    const positionLineNumber = position.line;
    if (positionLineNumber < tokenPosition.lineNumber) {
        return false;
    }
    else if (positionLineNumber > tokenPosition.lineNumber) {
        return true;
    }
    else {
        const upperBound = isBoundIncluded ? tokenPosition.lineCodeUnit : tokenPosition.lineCodeUnit - 1;
        return position.character > upperBound;
    }
}
exports.isAfterTokenPosition = isAfterTokenPosition;
//# sourceMappingURL=positionUtils.js.map