"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LanguageAutocompleteItemProvider = void 0;
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const keyword_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language/keyword/keyword");
const trace_1 = require("../trace");
class LanguageAutocompleteItemProvider {
    constructor(locale) {
        this.locale = locale;
    }
    getAutocompleteItems(context) {
        return __awaiter(this, void 0, void 0, function* () {
            // eslint-disable-next-line require-await
            return yield powerquery_parser_1.ResultUtils.ensureResultAsync(() => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const trace = context.traceManager.entry(trace_1.ProviderTraceConstant.LanguageCompletionProvider, this.getAutocompleteItems.name, context.initialCorrelationId);
                (_a = context.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
                const autocomplete = context.autocomplete;
                const autocompleteItems = [
                    ...this.getKeywords(autocomplete.triedKeyword, context.cancellationToken),
                    ...this.getLanguageConstants(autocomplete.triedLanguageConstant, context.cancellationToken),
                    ...this.getPrimitiveTypes(autocomplete.triedPrimitiveType, context.cancellationToken),
                ];
                trace.exit();
                return autocompleteItems;
            }), this.locale);
        });
    }
    getKeywords(triedKeywordAutocomplete, cancellationToken) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        if (powerquery_parser_1.ResultUtils.isError(triedKeywordAutocomplete)) {
            return [];
        }
        return triedKeywordAutocomplete.value.filter((autocompleteItem) => LanguageAutocompleteItemProvider.ExcludedKeywords.includes(autocompleteItem.label) === false);
    }
    getLanguageConstants(triedLanguageConstantAutocomplete, cancellationToken) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        return powerquery_parser_1.ResultUtils.isOk(triedLanguageConstantAutocomplete) ? triedLanguageConstantAutocomplete.value : [];
    }
    getPrimitiveTypes(triedPrimitiveTypeAutocomplete, cancellationToken) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        return powerquery_parser_1.ResultUtils.isOk(triedPrimitiveTypeAutocomplete) && triedPrimitiveTypeAutocomplete.value
            ? triedPrimitiveTypeAutocomplete.value
            : [];
    }
}
exports.LanguageAutocompleteItemProvider = LanguageAutocompleteItemProvider;
// Power Query defines constructor functions (ex. #table()) as keywords, but we want
// them to be treated like library functions instead.
LanguageAutocompleteItemProvider.ExcludedKeywords = [
    keyword_1.KeywordKind.HashBinary,
    keyword_1.KeywordKind.HashDate,
    keyword_1.KeywordKind.HashDateTime,
    keyword_1.KeywordKind.HashDateTimeZone,
    keyword_1.KeywordKind.HashDuration,
    keyword_1.KeywordKind.HashInfinity,
    keyword_1.KeywordKind.HashNan,
    keyword_1.KeywordKind.HashSections,
    keyword_1.KeywordKind.HashShared,
    keyword_1.KeywordKind.HashTable,
    keyword_1.KeywordKind.HashTime,
];
//# sourceMappingURL=languageCompletionItemProvider.js.map