"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFoldingRanges = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const trace_1 = require("../../trace");
function createFoldingRanges(nodeIdMapCollection, traceManager, correlationId) {
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, createFoldingRanges.name, correlationId);
    let foldingRanges = [];
    foldingRanges = foldingRanges
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.FunctionExpression, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.IfExpression, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.InvokeExpression, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.LetExpression, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.ListExpression, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.ListLiteral, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.MetadataExpression, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.RecordExpression, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.RecordLiteral, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.SectionMember, traceManager, trace.id))
        .concat(getFoldingRanges(nodeIdMapCollection, language_1.Ast.NodeKind.TypePrimaryType, traceManager, trace.id));
    trace.exit();
    return foldingRanges;
}
exports.createFoldingRanges = createFoldingRanges;
function getFoldingRanges(nodeIdMapCollection, nodeKind, traceManager, correlationId) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getFoldingRanges.name, correlationId, { nodeKind });
    const foldingRanges = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(nodeKind)) !== null && _a !== void 0 ? _a : []) {
        const node = parser_1.NodeIdMapUtils.astChecked(nodeIdMapCollection, nodeId, nodeKind);
        if (node === undefined) {
            continue;
        }
        const tokenRange = node.tokenRange;
        if (tokenRange.positionStart.lineNumber !== tokenRange.positionEnd.lineNumber) {
            foldingRanges.push(asFoldingRange(tokenRange));
        }
    }
    trace.exit();
    return foldingRanges;
}
function asFoldingRange(tokenRange) {
    return {
        startCharacter: tokenRange.positionStart.lineCodeUnit,
        startLine: tokenRange.positionStart.lineNumber,
        endCharacter: tokenRange.positionEnd.lineCodeUnit,
        endLine: tokenRange.positionEnd.lineNumber,
    };
}
//# sourceMappingURL=foldingRanges.js.map