import { CommonError, Result } from "@microsoft/powerquery-parser";
import { DocumentUri, FoldingRange, Hover, Location, SignatureHelp } from "vscode-languageserver-types";
import { AutocompleteItemProviderContext, DefinitionProviderContext, FoldingRangeProviderContext, HoverProviderContext, ILocalDocumentProvider, PartialSemanticToken, SemanticTokenProviderContext, SignatureProviderContext } from "../commonTypes";
import { Inspection } from "../..";
import { ILibrary } from "../../library/library";
export declare class LocalDocumentProvider implements ILocalDocumentProvider {
    private readonly uri;
    private readonly typeCache;
    private readonly library;
    protected readonly locale: string;
    constructor(uri: DocumentUri, typeCache: Inspection.TypeCache, library: ILibrary, locale: string);
    getAutocompleteItems(context: AutocompleteItemProviderContext): Promise<Result<Inspection.AutocompleteItem[] | undefined, CommonError.CommonError>>;
    getDefinition(context: DefinitionProviderContext): Promise<Result<Location[] | undefined, CommonError.CommonError>>;
    getFoldingRanges(context: FoldingRangeProviderContext): Promise<Result<FoldingRange[], CommonError.CommonError>>;
    getHover(context: HoverProviderContext): Promise<Result<Hover | undefined, CommonError.CommonError>>;
    getPartialSemanticTokens(context: SemanticTokenProviderContext): Promise<Result<PartialSemanticToken[] | undefined, CommonError.CommonError>>;
    getSignatureHelp(context: SignatureProviderContext): Promise<Result<SignatureHelp | undefined, CommonError.CommonError>>;
    protected getHoverForIdentifierPairedExpression(context: HoverProviderContext, correlationId: number): Promise<Hover | undefined>;
    protected getHoverForScopeItem(context: HoverProviderContext, nodeScope: Inspection.NodeScope, scopeType: Inspection.ScopeTypeByKey, correlationId: number): Hover | undefined;
    private getAutocompleteItemsFromFieldAccess;
}
