"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalDocumentProvider = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const InspectionUtils = require("../../inspectionUtils");
const __1 = require("../..");
const foldingRanges_1 = require("./foldingRanges");
const partialSemanticToken_1 = require("./partialSemanticToken");
const trace_1 = require("../../trace");
const inspection_1 = require("../../inspection");
class LocalDocumentProvider {
    constructor(uri, typeCache, library, locale) {
        this.uri = uri;
        this.typeCache = typeCache;
        this.library = library;
        this.locale = locale;
    }
    getAutocompleteItems(context) {
        // eslint-disable-next-line require-await
        return powerquery_parser_1.ResultUtils.ensureResultAsync(() => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const trace = context.traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, this.getAutocompleteItems.name, context.initialCorrelationId);
            (_a = context.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
            const autocompleteItems = [
                ...this.getAutocompleteItemsFromFieldAccess(context.autocomplete),
            ].concat(InspectionUtils.getAutocompleteItemsFromScope(context));
            trace.exit();
            return autocompleteItems;
        }), this.locale);
    }
    getDefinition(context) {
        // eslint-disable-next-line require-await
        return powerquery_parser_1.ResultUtils.ensureResultAsync(() => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const trace = context.traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, this.getDefinition.name, context.initialCorrelationId);
            (_a = context.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
            if (context.identifier.kind === language_1.Ast.NodeKind.GeneralizedIdentifier ||
                context.identifier.identifierContextKind !== language_1.Ast.IdentifierContextKind.Value) {
                return undefined;
            }
            const triedNodeScope = context.triedNodeScope;
            if (powerquery_parser_1.ResultUtils.isError(triedNodeScope)) {
                return undefined;
            }
            const scopeItem = triedNodeScope.value.get(context.identifier.literal);
            if (scopeItem === undefined) {
                return undefined;
            }
            const creator = inspection_1.ScopeUtils.scopeCreatorIdentifier(scopeItem);
            if (creator === undefined) {
                return undefined;
            }
            const location = {
                range: __1.PositionUtils.rangeFromTokenRange(creator.tokenRange),
                uri: this.uri,
            };
            trace.exit();
            return [location];
        }), this.locale);
    }
    getFoldingRanges(context) {
        // eslint-disable-next-line require-await
        return powerquery_parser_1.ResultUtils.ensureResultAsync(() => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const trace = context.traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, this.getFoldingRanges.name, context.initialCorrelationId);
            (_a = context.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
            const foldingRanges = (0, foldingRanges_1.createFoldingRanges)(context.nodeIdMapCollection, context.traceManager, trace.id);
            trace.exit();
            return foldingRanges;
        }), this.locale);
    }
    getHover(context) {
        return powerquery_parser_1.ResultUtils.ensureResultAsync(() => __awaiter(this, void 0, void 0, function* () {
            var _a;
            const trace = context.traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, this.getHover.name, context.initialCorrelationId);
            (_a = context.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
            let hover = yield this.getHoverForIdentifierPairedExpression(context, trace.id);
            if (hover !== undefined) {
                trace.exit({ hover: true });
                return hover;
            }
            const triedNodeScope = context.triedNodeScope;
            const triedScopeType = context.triedScopeType;
            if (!powerquery_parser_1.ResultUtils.isOk(triedNodeScope) || !powerquery_parser_1.ResultUtils.isOk(triedScopeType)) {
                trace.exit({ inspectionError: true });
                return undefined;
            }
            hover = this.getHoverForScopeItem(context, triedNodeScope.value, triedScopeType.value, trace.id);
            trace.exit();
            return hover !== null && hover !== void 0 ? hover : undefined;
        }), this.locale);
    }
    // eslint-disable-next-line require-await
    getPartialSemanticTokens(context) {
        return __awaiter(this, void 0, void 0, function* () {
            return powerquery_parser_1.ResultUtils.ensureResult(() => {
                var _a;
                const trace = context.traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, this.getPartialSemanticTokens.name, context.initialCorrelationId);
                (_a = context.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
                const nodeIdMapCollection = context.parseState.contextState.nodeIdMapCollection;
                const tokens = (0, partialSemanticToken_1.createPartialSemanticTokens)(nodeIdMapCollection, this.library.libraryDefinitions, context.traceManager, trace.id, context.cancellationToken);
                trace.exit();
                return tokens;
            }, this.locale);
        });
    }
    // eslint-disable-next-line require-await
    getSignatureHelp(context) {
        return __awaiter(this, void 0, void 0, function* () {
            return powerquery_parser_1.ResultUtils.ensureResult(() => {
                var _a;
                const trace = context.traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, this.getSignatureHelp.name, context.initialCorrelationId);
                (_a = context.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
                const invokeInspection = powerquery_parser_1.ResultUtils.isOk(context.triedCurrentInvokeExpression)
                    ? context.triedCurrentInvokeExpression.value
                    : undefined;
                if (invokeInspection === undefined) {
                    trace.exit({ invokeInspectionUndefined: true });
                    return undefined;
                }
                if (invokeInspection.name && !invokeInspection.isNameInLocalScope) {
                    trace.exit({ unknownName: true });
                    return undefined;
                }
                const identifierLiteral = context.functionName;
                if (identifierLiteral === undefined || !language_1.TypeUtils.isDefinedFunction(context.functionType)) {
                    return undefined;
                }
                const nameOfParameters = context.functionType.parameters
                    .map((parameter) => language_1.TypeUtils.nameOfFunctionParameter(parameter, context.traceManager, trace.id))
                    .join(", ");
                const label = `${identifierLiteral}(${nameOfParameters})`;
                const parameters = context.functionType.parameters;
                const result = {
                    activeParameter: context.argumentOrdinal,
                    activeSignature: 0,
                    signatures: [
                        {
                            label,
                            parameters: parameters.map((parameter) => ({
                                label: parameter.nameLiteral,
                            })),
                        },
                    ],
                };
                trace.exit();
                return result;
            }, this.locale);
        });
    }
    // When hovering over a key it should show the type for the value.
    // Covers:
    //  * GeneralizedIdentifierPairedAnyLiteral
    //  * GeneralizedIdentifierPairedExpression
    //  * IdentifierPairedExpression
    getHoverForIdentifierPairedExpression(context, correlationId) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const trace = context.traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, this.getHoverForIdentifierPairedExpression.name, correlationId);
            (_a = context.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
            const ancestry = context.activeNode.ancestry;
            const leafKInd = (_b = ancestry[0]) === null || _b === void 0 ? void 0 : _b.node.kind;
            const isValidLeafNodeKind = [language_1.Ast.NodeKind.GeneralizedIdentifier, language_1.Ast.NodeKind.Identifier].includes(leafKInd);
            if (!isValidLeafNodeKind) {
                trace.exit();
                return undefined;
            }
            const identifierPairedExpression = parser_1.AncestryUtils.nthChecked(context.activeNode.ancestry, 1, [
                language_1.Ast.NodeKind.GeneralizedIdentifierPairedAnyLiteral,
                language_1.Ast.NodeKind.GeneralizedIdentifierPairedExpression,
                language_1.Ast.NodeKind.IdentifierPairedExpression,
            ]);
            // We're on an identifier in some other context which we don't support.
            if (identifierPairedExpression === undefined) {
                trace.exit();
                return undefined;
            }
            const expression = parser_1.NodeIdMapUtils.nthChildXor(context.parseState.contextState.nodeIdMapCollection, identifierPairedExpression.node.id, 2);
            // We're on an identifier in some other context which we don't support.
            if (expression === undefined) {
                trace.exit();
                return undefined;
            }
            const triedExpressionType = yield __1.Inspection.tryType(Object.assign(Object.assign({}, context.inspectionSettings), { initialCorrelationId: trace.id }), context.parseState.contextState.nodeIdMapCollection, expression.node.id, this.typeCache);
            // TODO handle error
            if (powerquery_parser_1.ResultUtils.isError(triedExpressionType)) {
                trace.exit();
                return undefined;
            }
            let scopeItemText = "unknown";
            // If it's a SectionMember
            const thirdNodeKind = (_c = ancestry[2]) === null || _c === void 0 ? void 0 : _c.node.kind;
            if (thirdNodeKind === language_1.Ast.NodeKind.SectionMember) {
                scopeItemText = InspectionUtils.getScopeItemKindText(__1.Inspection.ScopeItemKind.SectionMember);
            }
            // Else if it's RecordExpression or RecordLiteral
            const fifthNodeKind = (_d = ancestry[4]) === null || _d === void 0 ? void 0 : _d.node.kind;
            const isRecordNodeKind = [language_1.Ast.NodeKind.RecordExpression, language_1.Ast.NodeKind.RecordLiteral].includes(fifthNodeKind);
            if (isRecordNodeKind) {
                scopeItemText = InspectionUtils.getScopeItemKindText(__1.Inspection.ScopeItemKind.RecordField);
            }
            else if (fifthNodeKind === language_1.Ast.NodeKind.LetExpression) {
                scopeItemText = InspectionUtils.getScopeItemKindText(__1.Inspection.ScopeItemKind.LetVariable);
            }
            const nameOfExpressionType = language_1.TypeUtils.nameOf(triedExpressionType.value, context.traceManager, trace.id);
            const result = {
                contents: {
                    kind: vscode_languageserver_types_1.MarkupKind.PlainText,
                    language: "powerquery",
                    value: `[${scopeItemText}] ${context.identifier.literal}: ${nameOfExpressionType}`,
                },
                range: undefined,
            };
            trace.exit();
            return result;
        });
    }
    getHoverForScopeItem(context, nodeScope, scopeType, correlationId) {
        const identifierLiteral = context.identifier.literal;
        const scopeItem = nodeScope.get(identifierLiteral);
        if (scopeItem === undefined || scopeItem.kind === __1.Inspection.ScopeItemKind.Undefined) {
            return undefined;
        }
        const scopeItemText = InspectionUtils.getScopeItemKindText(scopeItem.kind);
        const scopeItemType = scopeType.get(identifierLiteral);
        const scopeItemTypeText = scopeItemType !== undefined
            ? language_1.TypeUtils.nameOf(scopeItemType, context.traceManager, correlationId)
            : "unknown";
        return {
            contents: {
                kind: vscode_languageserver_types_1.MarkupKind.PlainText,
                language: "powerquery",
                value: `[${scopeItemText}] ${identifierLiteral}: ${scopeItemTypeText}`,
            },
            range: undefined,
        };
    }
    getAutocompleteItemsFromFieldAccess(autocomplete) {
        const triedFieldAccess = autocomplete.triedFieldAccess;
        return powerquery_parser_1.ResultUtils.isOk(triedFieldAccess) && triedFieldAccess.value !== undefined
            ? triedFieldAccess.value.autocompleteItems
            : [];
    }
}
exports.LocalDocumentProvider = LocalDocumentProvider;
//# sourceMappingURL=localDocumentProvider.js.map