"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPartialSemanticTokens = void 0;
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const language_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/language");
const __1 = require("../..");
const trace_1 = require("../../trace");
function createPartialSemanticTokens(nodeIdMapCollection, libraryDefinitions, traceManager, correlationId, cancellationToken) {
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, createPartialSemanticTokens.name, correlationId);
    let tokens = [];
    // Consumed as first-come first-serve priority.
    tokens = tokens
        .concat(getAsNullablePrimitiveTypeTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getFieldSelectorTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getFieldProjectionTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getFieldSpecificationTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getGeneralizedIdentifierPairedAnyLiteralTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getGeneralizedIdentifierPairedExpressionTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getIdentifierPairedExpressionTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getInvokeExpressionTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getLiteralTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getNullablePrimitiveTypeTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getNullableTypeTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getParameterTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getPrimitiveTypeTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getTBinOpExpressionTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getTableTypeTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        .concat(getTypePrimaryTypeTokens(nodeIdMapCollection, traceManager, trace.id, cancellationToken))
        // Should be the lowest priority
        .concat(getIdentifierExpressionTokens(nodeIdMapCollection, libraryDefinitions, traceManager, trace.id, cancellationToken));
    trace.exit();
    return tokens;
}
exports.createPartialSemanticTokens = createPartialSemanticTokens;
function getAsNullablePrimitiveTypeTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getAsNullablePrimitiveTypeTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.AsNullablePrimitiveType)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 0, tokens, vscode_languageserver_types_1.SemanticTokenTypes.keyword);
    }
    trace.exit();
    return tokens;
}
function getFieldSelectorTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getFieldSelectorTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.FieldSelector)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.GeneralizedIdentifier, 1, tokens, vscode_languageserver_types_1.SemanticTokenTypes.variable);
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 3, tokens, vscode_languageserver_types_1.SemanticTokenTypes.operator);
    }
    trace.exit();
    return tokens;
}
function getFieldProjectionTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getFieldProjectionTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.FieldProjection)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 3, tokens, vscode_languageserver_types_1.SemanticTokenTypes.operator);
    }
    trace.exit();
    return tokens;
}
function getFieldSpecificationTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getFieldSpecificationTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.FieldSpecification)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 0, tokens, vscode_languageserver_types_1.SemanticTokenTypes.keyword);
    }
    trace.exit();
    return tokens;
}
function getGeneralizedIdentifierPairedExpressionTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    return getPairedExpressionTokens(nodeIdMapCollection, language_1.Ast.NodeKind.GeneralizedIdentifierPairedExpression, language_1.Ast.NodeKind.GeneralizedIdentifier, traceManager, correlationId, cancellationToken);
}
function getGeneralizedIdentifierPairedAnyLiteralTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    return getPairedExpressionTokens(nodeIdMapCollection, language_1.Ast.NodeKind.GeneralizedIdentifierPairedAnyLiteral, language_1.Ast.NodeKind.GeneralizedIdentifier, traceManager, correlationId, cancellationToken);
}
function getIdentifierExpressionTokens(nodeIdMapCollection, libraryDefinitions, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getIdentifierExpressionTokens.name, correlationId);
    const tokens = [];
    for (const identifierId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.IdentifierExpression)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        const identifierExpr = nodeIdMapCollection.astNodeById.get(identifierId);
        let tokenType;
        let tokenModifiers = [];
        switch (identifierExpr === null || identifierExpr === void 0 ? void 0 : identifierExpr.identifier.identifierContextKind) {
            case language_1.Ast.IdentifierContextKind.Key:
                tokenType = vscode_languageserver_types_1.SemanticTokenTypes.property;
                tokenModifiers = [vscode_languageserver_types_1.SemanticTokenModifiers.declaration];
                break;
            case language_1.Ast.IdentifierContextKind.Parameter:
                tokenType = vscode_languageserver_types_1.SemanticTokenTypes.parameter;
                break;
            case language_1.Ast.IdentifierContextKind.Value:
                tokenType = vscode_languageserver_types_1.SemanticTokenTypes.variable;
                if (libraryDefinitions.staticLibraryDefinitions.has(identifierExpr.identifier.literal)) {
                    tokenModifiers = [vscode_languageserver_types_1.SemanticTokenModifiers.defaultLibrary];
                }
                break;
            default:
                continue;
        }
        tokens.push({
            range: __1.PositionUtils.rangeFromTokenRange(identifierExpr.tokenRange),
            tokenModifiers,
            tokenType,
        });
    }
    trace.exit();
    return tokens;
}
function getIdentifierPairedExpressionTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    return getPairedExpressionTokens(nodeIdMapCollection, language_1.Ast.NodeKind.IdentifierPairedExpression, language_1.Ast.NodeKind.Identifier, traceManager, correlationId, cancellationToken);
}
function getInvokeExpressionTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getInvokeExpressionTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.InvokeExpression)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        const identifierXor = parser_1.NodeIdMapUtils.invokeExpressionIdentifier(nodeIdMapCollection, nodeId);
        if (identifierXor === undefined || !parser_1.XorNodeUtils.isAst(identifierXor)) {
            continue;
        }
        tokens.push({
            range: __1.PositionUtils.rangeFromTokenRange(identifierXor.node.tokenRange),
            tokenModifiers: [],
            tokenType: vscode_languageserver_types_1.SemanticTokenTypes.function,
        });
    }
    trace.exit();
    return tokens;
}
function getLiteralTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getLiteralTokens.name, correlationId);
    const tokens = [];
    for (const literalId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.LiteralExpression)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        const literal = nodeIdMapCollection.astNodeById.get(literalId);
        let tokenType;
        switch (literal === null || literal === void 0 ? void 0 : literal.literalKind) {
            case language_1.Ast.LiteralKind.Numeric:
                tokenType = vscode_languageserver_types_1.SemanticTokenTypes.number;
                break;
            case language_1.Ast.LiteralKind.Text:
                tokenType = vscode_languageserver_types_1.SemanticTokenTypes.string;
                break;
            default:
                continue;
        }
        tokens.push({
            range: __1.PositionUtils.rangeFromTokenRange(literal.tokenRange),
            tokenModifiers: [],
            tokenType,
        });
    }
    trace.exit();
    return tokens;
}
function getNullablePrimitiveTypeTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getNullablePrimitiveTypeTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.NullablePrimitiveType)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 0, tokens, vscode_languageserver_types_1.SemanticTokenTypes.keyword);
    }
    trace.exit();
    return tokens;
}
function getNullableTypeTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getNullableTypeTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.NullableType)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 0, tokens, vscode_languageserver_types_1.SemanticTokenTypes.keyword);
    }
    trace.exit();
    return tokens;
}
function getParameterTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getParameterTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.Parameter)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 0, tokens, vscode_languageserver_types_1.SemanticTokenTypes.keyword);
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Identifier, 1, tokens, vscode_languageserver_types_1.SemanticTokenTypes.parameter, [
            vscode_languageserver_types_1.SemanticTokenModifiers.declaration,
        ]);
    }
    trace.exit();
    return tokens;
}
function getPrimitiveTypeTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getPrimitiveTypeTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.PrimitiveType)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        const node = parser_1.NodeIdMapUtils.astChecked(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.PrimitiveType);
        if (node === undefined) {
            continue;
        }
        tokens.push({
            range: __1.PositionUtils.rangeFromTokenRange(node.tokenRange),
            tokenModifiers: [],
            tokenType: vscode_languageserver_types_1.SemanticTokenTypes.type,
        });
    }
    trace.exit();
    return tokens;
}
function getTBinOpExpressionTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getTBinOpExpressionTokens.name, correlationId);
    const binOpExprNodeKinds = [
        language_1.Ast.NodeKind.ArithmeticExpression,
        language_1.Ast.NodeKind.AsExpression,
        language_1.Ast.NodeKind.EqualityExpression,
        language_1.Ast.NodeKind.IsExpression,
        language_1.Ast.NodeKind.LogicalExpression,
        language_1.Ast.NodeKind.MetadataExpression,
        language_1.Ast.NodeKind.NullCoalescingExpression,
        language_1.Ast.NodeKind.RelationalExpression,
    ];
    const tokens = [];
    for (const nodeKind of binOpExprNodeKinds) {
        for (const binOpExprId of (_a = nodeIdMapCollection.idsByNodeKind.get(nodeKind)) !== null && _a !== void 0 ? _a : []) {
            cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
            const exprXorNode = parser_1.NodeIdMapUtils.xor(nodeIdMapCollection, binOpExprId);
            if (exprXorNode === undefined) {
                continue;
            }
            const operatorNode = parser_1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, exprXorNode.node.id, 1, language_1.Ast.NodeKind.Constant);
            if (operatorNode === undefined) {
                continue;
            }
            const tokenType = [
                language_1.Ast.NodeKind.AsExpression,
                language_1.Ast.NodeKind.IsExpression,
                language_1.Ast.NodeKind.MetadataExpression,
            ].includes(exprXorNode.node.kind)
                ? vscode_languageserver_types_1.SemanticTokenTypes.keyword
                : vscode_languageserver_types_1.SemanticTokenTypes.operator;
            tokens.push({
                range: __1.PositionUtils.rangeFromTokenRange(operatorNode.tokenRange),
                tokenModifiers: [],
                tokenType,
            });
        }
    }
    trace.exit();
    return tokens;
}
function getTableTypeTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getTableTypeTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.TableType)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 0, tokens, vscode_languageserver_types_1.SemanticTokenTypes.type);
    }
    trace.exit();
    return tokens;
}
function getTypePrimaryTypeTokens(nodeIdMapCollection, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getTypePrimaryTypeTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(language_1.Ast.NodeKind.TypePrimaryType)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        pushNthChild(nodeIdMapCollection, nodeId, language_1.Ast.NodeKind.Constant, 0, tokens, vscode_languageserver_types_1.SemanticTokenTypes.type);
    }
    trace.exit();
    return tokens;
}
function getPairedExpressionTokens(nodeIdMapCollection, pairedKind, keyKind, traceManager, correlationId, cancellationToken) {
    var _a;
    const trace = traceManager.entry(trace_1.ProviderTraceConstant.LocalDocumentSymbolProvider, getIdentifierExpressionTokens.name, correlationId);
    const tokens = [];
    for (const nodeId of (_a = nodeIdMapCollection.idsByNodeKind.get(pairedKind)) !== null && _a !== void 0 ? _a : []) {
        cancellationToken === null || cancellationToken === void 0 ? void 0 : cancellationToken.throwIfCancelled();
        const keyNode = parser_1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, nodeId, 0, keyKind);
        if (keyNode === undefined) {
            continue;
        }
        tokens.push({
            range: __1.PositionUtils.rangeFromTokenRange(keyNode.tokenRange),
            tokenModifiers: [vscode_languageserver_types_1.SemanticTokenModifiers.declaration],
            tokenType: vscode_languageserver_types_1.SemanticTokenTypes.variable,
        });
    }
    trace.exit();
    return tokens;
}
function pushNthChild(nodeIdMapCollection, parentId, childNodeKind, attributeIndex, tokens, tokenType, tokenModifiers = []) {
    const typeConstant = parser_1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, parentId, attributeIndex, childNodeKind);
    if (typeConstant === undefined) {
        return;
    }
    tokens.push({
        range: __1.PositionUtils.rangeFromTokenRange(typeConstant.tokenRange),
        tokenModifiers,
        tokenType,
    });
}
//# sourceMappingURL=partialSemanticToken.js.map