"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validate = void 0;
const analysis_1 = require("../analysis");
const powerquery_parser_1 = require("@microsoft/powerquery-parser");
const validateDuplicateIdentifiers_1 = require("./validateDuplicateIdentifiers");
const validateFunctionExpression_1 = require("./validateFunctionExpression");
const validateInvokeExpression_1 = require("./validateInvokeExpression");
const validateParse_1 = require("./validateParse");
const validateUnknownIdentifiers_1 = require("./validateUnknownIdentifiers");
const trace_1 = require("../trace");
function validate(textDocument, analysisSettings, validationSettings) {
    return powerquery_parser_1.ResultUtils.ensureResultAsync(() => __awaiter(this, void 0, void 0, function* () {
        const trace = validationSettings.traceManager.entry(trace_1.ValidationTraceConstant.Validation, validate.name, validationSettings.initialCorrelationId);
        const updatedSettings = Object.assign(Object.assign({}, validationSettings), { initialCorrelationId: trace.id });
        const analysis = analysis_1.AnalysisUtils.analysis(textDocument, analysisSettings);
        const parseState = powerquery_parser_1.ResultUtils.assertOk(yield analysis.getParseState());
        const parseError = powerquery_parser_1.ResultUtils.assertOk(yield analysis.getParseError());
        if (parseState === undefined) {
            trace.exit();
            return undefined;
        }
        let functionExpressionDiagnostics;
        let invokeExpressionDiagnostics;
        let unknownIdentifiersDiagnostics;
        const nodeIdMapCollection = parseState.contextState.nodeIdMapCollection;
        const typeCache = analysis.getTypeCache();
        if (validationSettings.checkInvokeExpressions && nodeIdMapCollection) {
            functionExpressionDiagnostics = (0, validateFunctionExpression_1.validateFunctionExpression)(validationSettings, nodeIdMapCollection);
            invokeExpressionDiagnostics = yield (0, validateInvokeExpression_1.validateInvokeExpression)(validationSettings, nodeIdMapCollection, typeCache);
        }
        else {
            functionExpressionDiagnostics = [];
            invokeExpressionDiagnostics = [];
        }
        if (validationSettings.checkUnknownIdentifiers && nodeIdMapCollection) {
            unknownIdentifiersDiagnostics = yield (0, validateUnknownIdentifiers_1.validateUnknownIdentifiers)(validationSettings, nodeIdMapCollection, typeCache);
        }
        else {
            unknownIdentifiersDiagnostics = [];
        }
        const result = {
            diagnostics: [
                ...(0, validateDuplicateIdentifiers_1.validateDuplicateIdentifiers)(textDocument, nodeIdMapCollection, updatedSettings, validationSettings.cancellationToken),
                ...(yield (0, validateParse_1.validateParse)(parseError, updatedSettings)),
                ...functionExpressionDiagnostics,
                ...invokeExpressionDiagnostics,
                ...unknownIdentifiersDiagnostics,
            ],
            hasSyntaxError: parseError !== undefined,
        };
        trace.exit();
        return result;
    }), validationSettings.locale);
}
exports.validate = validate;
//# sourceMappingURL=validate.js.map