"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateParse = void 0;
const PQP = require("@microsoft/powerquery-parser");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
const parser_1 = require("@microsoft/powerquery-parser/lib/powerquery-parser/parser");
const diagnosticErrorCode_1 = require("../diagnosticErrorCode");
function validateParse(parseError, validationSettings) {
    return __awaiter(this, void 0, void 0, function* () {
        if (parseError === undefined || !parser_1.ParseError.isParseError(parseError)) {
            return [];
        }
        const innerError = parseError.innerError;
        const errorToken = PQP.Parser.ParseError.tokenFrom(innerError);
        const message = parseError.message;
        const parseContextState = parseError.state.contextState;
        let range;
        if (errorToken !== undefined) {
            range = {
                start: {
                    line: errorToken.positionStart.lineNumber,
                    character: errorToken.positionStart.lineCodeUnit,
                },
                end: {
                    line: errorToken.positionEnd.lineNumber,
                    character: errorToken.positionEnd.lineCodeUnit,
                },
            };
        }
        else {
            const root = parseContextState.root;
            if (root === undefined) {
                return [];
            }
            const leaf = yield parser_1.NodeIdMapUtils.rightMostLeaf(parseError.state.contextState.nodeIdMapCollection, root.id);
            if (leaf === undefined) {
                return [];
            }
            const leafTokenRange = leaf.tokenRange;
            range = {
                start: {
                    line: leafTokenRange.positionStart.lineNumber,
                    character: leafTokenRange.positionStart.lineCodeUnit,
                },
                end: {
                    line: leafTokenRange.positionEnd.lineNumber,
                    character: leafTokenRange.positionEnd.lineCodeUnit,
                },
            };
        }
        return [
            {
                code: diagnosticErrorCode_1.DiagnosticErrorCode.ParseError,
                message,
                range,
                severity: vscode_languageserver_types_1.DiagnosticSeverity.Error,
                source: validationSettings === null || validationSettings === void 0 ? void 0 : validationSettings.source,
            },
        ];
    });
}
exports.validateParse = validateParse;
//# sourceMappingURL=validateParse.js.map